// DataToDisplayManager.cpp: implementation of the DataToDisplayManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DataDisplay.h"
#include "DataToDisplayManager.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#include "..\shared\filters_processing\filter_load.h"
#include "..\shared\filters_display\filter_normaldisp.h"
#include "..\shared\displays\display_calcbounds.h"

class CDCDisplayInterface:public DisplayInterface<GPSPoint>
{
	ChildViewMode CVMode;//
	CDC *dc;

	ModeDisplayInfo DData;
	RECT DRect;
	COLORTYPE DefaultColor;
	public:
	
	CDCDisplayInterface(CDC &Idc,ChildViewMode Mode,ModeDisplayInfo  DD,RECT Rect) {dc=&Idc;CVMode=Mode;DData=DD;DRect=Rect;DefaultColor=0x008080ff;}

	CDCDisplayInterface(ChildViewMode Mode,ModeDisplayInfo  DD,RECT Rect) {dc=NULL;CVMode=Mode;DData=DD;DRect=Rect;} //mozno pouzit v pripade pouziti kounverznich funkci


	virtual POINTf GetScreenPos(const GPSPoint &Value);
    virtual void DrawLine(const GPSPoint &A,const GPSPoint &B,COLORTYPE Color);
    virtual void DrawLine(const GPSPoint &A,const GPSPoint &B);
	
	void SetDefaultColor(COLORTYPE DC) {DefaultColor=DC;}

	POINTf GetSlozky(const GPSPoint &Value);
	GPSPoint SetSlozky(const POINTf &Value);
};

POINTf CDCDisplayInterface::GetSlozky(const GPSPoint &Value)
{
	POINTf Output;
	VECT3D V=(VECT3D)Value;
	switch (CVMode)
	{
		case CV_MODE_SD:Output.y=V.s;Output.x=V.d;break;
		case CV_MODE_SV:Output.x=V.s;Output.y=V.v;break;
		case CV_MODE_DV:Output.x=V.d;Output.y=V.v;break;
		case CV_MODE_LS:Output.x=Value.DelkaNaKrivce;Output.y=V.s;break;
		case CV_MODE_LD:Output.x=Value.DelkaNaKrivce;Output.y=V.d;break;
		case CV_MODE_LV:Output.x=Value.DelkaNaKrivce;Output.y=V.v;break;
		case CV_MODE_NLS:Output.x=Value.DelkaNaKrivce/Value.DelkaKrivky;Output.y=V.s;break;
		case CV_MODE_NLD:Output.x=Value.DelkaNaKrivce/Value.DelkaKrivky;Output.y=V.d;break;
		case CV_MODE_NLV:Output.x=Value.DelkaNaKrivce/Value.DelkaKrivky;Output.y=V.v;break;
	}
	return Output;
}

GPSPoint CDCDisplayInterface::SetSlozky(const POINTf &Value)
{
	VECT3D V(0,0,0);
	GPSPoint Output;
	switch (CVMode)
	{
		case CV_MODE_SD:V.s=Value.y;V.d=Value.x;break;
		case CV_MODE_SV:V.s=Value.x;V.v=Value.y;break;
		case CV_MODE_DV:V.d=Value.x;V.v=Value.y;break;
		case CV_MODE_LS:Output.DelkaNaKrivce=Value.x;V.s=Value.y;break;
		case CV_MODE_LD:Output.DelkaNaKrivce=Value.x;V.d=Value.y;break;
		case CV_MODE_LV:Output.DelkaNaKrivce=Value.x;V.v=Value.y;break;
		case CV_MODE_NLS:Output.DelkaNaKrivce=Value.x;V.s=Value.y;break;
		case CV_MODE_NLD:Output.DelkaNaKrivce=Value.x;V.d=Value.y;break;
		case CV_MODE_NLV:Output.DelkaNaKrivce=Value.x;V.v=Value.y;break;
	}
	Output.InitByMetry(V);
	return Output;

}

POINTf CDCDisplayInterface::GetScreenPos(const GPSPoint &Value)
{
	POINTf Output=GetSlozky(Value);
	Output.x+=DData.Delta.x;
	Output.y+=DData.Delta.y;
	Output.x*=DData.Scale.x;
	Output.y*=DData.Scale.y;
	Output.y=DRect.bottom-Output.y;
	return Output;
}

void CDCDisplayInterface::DrawLine(const GPSPoint &A,const GPSPoint &B,COLORTYPE Color)
{
	POINTf Apos=GetScreenPos(A);
	POINTf Bpos=GetScreenPos(B);
	POINT AIntPos;
	POINT BIntPos;
	AIntPos.x=(int)Apos.x;
	AIntPos.y=(int)Apos.y;
	BIntPos.x=(int)Bpos.x;
	BIntPos.y=(int)Bpos.y;

	CPen NewPen(PS_SOLID,0,COLORREF(Color));	
	CBrush NewBrush;
	NewBrush.CreateSolidBrush(COLORREF(Color));

	CPen *OldPen=dc->SelectObject(&NewPen);	
	CBrush *OldBrush=dc->SelectObject((CBrush *)&NewBrush);

	dc->MoveTo(AIntPos);
	dc->LineTo(BIntPos);

	dc->SelectObject(OldBrush);
	dc->SelectObject(OldPen);
}

void CDCDisplayInterface::DrawLine(const GPSPoint &A,const GPSPoint &B)
{
	DrawLine(A,B,DefaultColor);
}

void DataToDisplayManager::ClearListOfLists(void)
{
	DataToView.clear();
}

DisplayingOfProcessingFiltersListGPSPoint *DataToDisplayManager::CreateList(RefCounterHolder<ProcessingFiltersList<GPSPoint> > List)
{
	DisplayingOfProcessingFiltersListGPSPoint  *PFL=new DisplayingOfProcessingFiltersListGPSPoint;
	DisplayingOfProcessingFiltersList<GPSPoint>  *PFL2=new DisplayingOfProcessingFiltersList<GPSPoint>;
	PFL->SetTo(PFL2);
	PFL2->SetList(List);
	PFL2->SetDisplayFilter(new NormalDisplayFilter);
	PFL2->SetInputBuffer(NULL);
	return PFL;
}

void DataToDisplayManager::AddList(DisplayingOfProcessingFiltersListGPSPoint *List,COLORREF Color)
{
	SingleListItem ITM;
	ITM.GraphToDraw=List;
	ITM.DisplayColor=Color;
	DataToView.push_back(ITM);
}

void DataToDisplayManager::SetLengthMeasureStart(GPSPoint *NewStart)
{
	LengthMeasureStartSet=false;
	if (NewStart==NULL) return;
	LengthMeasureStartSet=true;
	LengthMeasureStart=*NewStart;
}

const GPSPoint *DataToDisplayManager::GetLengthMeasureStart(void)
{
	if (!LengthMeasureStartSet) return NULL;
	return &LengthMeasureStart;
}


DataToDisplayManager::DataToDisplayManager()
{
	CurrentMode=CV_MODE_SD;
	LengthMeasureStartSet=false;
}

DataToDisplayManager::~DataToDisplayManager()
{

}

ChildViewMode DataToDisplayManager::GetViewMode()
{
	return CurrentMode; 
}

void DataToDisplayManager::SetViewMode(ChildViewMode NewMode)
{
	CurrentMode=NewMode;
	if (!DataProMody[CurrentMode].Inited && DataToView.size())
		ScaleToFit(false);		
}
void DataToDisplayManager::SetImageRect(RECT &r)
{
	RectangleOnScreen=r;
}

void DataToDisplayManager::InvalidateLists(void)
{
	listoflistoffilters::iterator it;
	for (it=DataToView.begin();it!=DataToView.end();it++)
		it->GraphToDraw->GetValue()->Recalculate();
	
	listofmatchingfilters::iterator it2;
	for (it2=MatchesToView.begin();it2!=MatchesToView.end();it2++)
		it2->Reset();
}


void DataToDisplayManager::DrawContents(CDC &dc)
{
	CBrush Black(COLORREF(0));
	dc.FillRect(&RectangleOnScreen,&Black); //nemame data->vyprazdnime obrazovku

	RefCounterHolder<CDCDisplayInterface> CDCI=new CDCDisplayInterface(dc,CurrentMode,DataProMody[CurrentMode],RectangleOnScreen);

	//spocitame grafy
	listoflistoffilters::iterator it;
	for (it=DataToView.begin();it!=DataToView.end();it++)
		it->GraphToDraw->GetValue()->CalculateIfNotCached();


	RefCounterHolder<DisplayingFilter<GPSPoint> >DFilter=new NormalDisplayFilter;
	//vykreslime vysledky matchovani
	listofmatchingfilters::iterator it2;
	for (it2=MatchesToView.begin();it2!=MatchesToView.end();it2++)
	{
		CDCI->SetDefaultColor(it2->Color);		
		it2->Recalculate();
		list<SingleMatch>::iterator it3;

		if (it2->Color!=0) //cernou barvu ingorujeme
			for (it3=it2->SecondaryTracks.begin();it3!=it2->SecondaryTracks.end();it3++)
				DFilter->DisplayCrossBuffer(*(it3->CachedResult),CDCI.GetValue());
	}

	//vykreslime zdrojove krivky
	for (it=DataToView.begin();it!=DataToView.end();it++)
	{
		CDCI->SetDefaultColor(it->DisplayColor);
		it->GraphToDraw->GetValue()->Redraw(CDCI.GetValue());
	}

	//vykreslime vysledne krivky
	for (it2=MatchesToView.begin();it2!=MatchesToView.end();it2++)
	{
		if (it2->MergeFilter.GetValue()==NULL) continue;
		{
			if (it2->MergeColor!=0) //cernou barvu ignorujeme
			{				
				if (it2->CachedMerge->IsNULL())
				{	
					GPSPointBuffer::Holder ResultBuff;
					Buffer<MatchPair<GPSPoint> >::Holder Matches=new Buffer<MatchPair<GPSPoint> >;

					list<SingleMatch>::iterator it3;
					for (it3=it2->SecondaryTracks.begin();it3!=it2->SecondaryTracks.end();it3++)
					{
						MatchPair<GPSPoint> Mp;
						Mp.Curve=it3->SecondaryTrackGraphMergeGen->GetValue()->GetCachedBuffer();
						Mp.MatchData=*(it3->CachedResult);
						Matches->Data.push_back(Mp);
					}		
					it2->MergeFilter->GenerateResult(it2->PrimaryTrackMergeGen->GetValue()->GetCachedBuffer(),Matches,ResultBuff);
					it2->CachedMerge->SetTo(dynamic_cast<GPSPointBuffer *>(ResultBuff.GetValue()));
				}
				CDCI->SetDefaultColor(it2->MergeColor);
				if (!it2->CachedMerge->IsNULL())
					DFilter->DisplayBuffer(GPSPointBuffer::Holder(it2->CachedMerge->GetValue()),CDCI.GetValue());
			}
		}
	}

	if (LengthMeasureStartSet && CurrentMode==CV_MODE_SD)
	{
		GPSPoint KrizekWidht,KrizekHeight;

		KrizekWidht.InitByMetry(VECT3D(0,0.5,0));
		KrizekHeight.InitByMetry(VECT3D(0.5,0,0));
		GPSPoint LeftPoint=LengthMeasureStart;
		GPSPoint RightPoint=LengthMeasureStart;
		LeftPoint-=KrizekWidht;
		RightPoint+=KrizekWidht;

		GPSPoint TopPoint=LengthMeasureStart;
		GPSPoint DownPoint=LengthMeasureStart;

		TopPoint+=KrizekHeight;
		DownPoint-=KrizekHeight;
		
		CDCI->SetDefaultColor(0xff80ff);
		CDCI->DrawLine(LeftPoint,RightPoint);
		CDCI->DrawLine(TopPoint,DownPoint);
	}



/*
	//test jestli GenerateResultCurveForSelectedMatch opravdu funguje
	SingleMatchingTrack *t=GlobalDataSpace.GetMatchById(1);
	GPSPointBuffer::Holder Result= GlobalDataSpace.GenerateResultCurveForSelectedMatch(t);
	CDCI->SetDefaultColor(0xff0000);
	DFilter->DisplayBuffer(Result.GetValue(),CDCI.GetValue());*/

}

void DataToDisplayManager::ScaleToFit(bool Proportional)
{
	if (DataToView.size()==0) return;
	//spocitame bounding box objektu
	RefCounterHolder<Display_CalcBounds> DCB=new Display_CalcBounds;


	CDCDisplayInterface TmpCDC(CurrentMode,DataProMody[CurrentMode],RectangleOnScreen);
	POINTf MinV(0,0);
	POINTf MaxV(0,0);
	listoflistoffilters::iterator it;
	for (it=DataToView.begin();it!=DataToView.end();it++)
	{

		it->GraphToDraw->GetValue()->Redraw(DCB.GetValue());
		
		if (it==DataToView.begin())
		{
			MinV=TmpCDC.GetSlozky(DCB->GetMin());
			MaxV=TmpCDC.GetSlozky(DCB->GetMax());
		}
		else
		{
			POINTf TmpMinV=TmpCDC.GetSlozky(DCB->GetMin());
			POINTf TmpMaxV=TmpCDC.GetSlozky(DCB->GetMax());
			if (TmpMinV.x<MinV.x) MinV.x=TmpMinV.x;
			if (TmpMinV.y<MinV.y) MinV.y=TmpMinV.y;
			if (TmpMaxV.x>MaxV.x) MaxV.x=TmpMaxV.x;
			if (TmpMaxV.y>MaxV.y) MaxV.y=TmpMaxV.y;
		}
	}


	DataProMody[CurrentMode].Delta.x=-MinV.x;
	DataProMody[CurrentMode].Delta.y=-MinV.y;

	if (RectangleOnScreen.right!=RectangleOnScreen.left)
		DataProMody[CurrentMode].Scale.x=(FTYPE)(RectangleOnScreen.right-RectangleOnScreen.left)/(MaxV.x-MinV.x);
	else
		DataProMody[CurrentMode].Scale.x=1.0f;

	if (RectangleOnScreen.bottom!=RectangleOnScreen.top)
		DataProMody[CurrentMode].Scale.y=(FTYPE)(RectangleOnScreen.bottom-RectangleOnScreen.top)/(MaxV.y-MinV.y);
	else
		DataProMody[CurrentMode].Scale.y=1.0f;


	if (Proportional)
	{
		FTYPE MinScale=DataProMody[CurrentMode].Scale.x;
		if (MinScale>DataProMody[CurrentMode].Scale.y) MinScale=DataProMody[CurrentMode].Scale.y;
		DataProMody[CurrentMode].Scale.y=DataProMody[CurrentMode].Scale.x=MinScale;
	}

	DataProMody[CurrentMode].Inited=true;
}

void DataToDisplayManager::Move(POINT &p1, POINT &p2)
{
	DataProMody[CurrentMode].Delta.x+=(p2.x-p1.x)/DataProMody[CurrentMode].Scale.x;
	DataProMody[CurrentMode].Delta.y+=(p2.y-p1.y)/DataProMody[CurrentMode].Scale.y;
	DataProMody[CurrentMode].Inited=true;
}

void DataToDisplayManager::Scale(POINT &p1, POINT &p2,bool KeepAspect)
{
	FTYPE InRealSpacePosOfP1=(p1.x/DataProMody[CurrentMode].Scale.x)-DataProMody[CurrentMode].Delta.x;
	FTYPE NoveX=0.0f;
	if (InRealSpacePosOfP1+DataProMody[CurrentMode].Delta.x!=0) NoveX=p2.x/(InRealSpacePosOfP1+DataProMody[CurrentMode].Delta.x);	
	
	InRealSpacePosOfP1=(p1.y/DataProMody[CurrentMode].Scale.y)-DataProMody[CurrentMode].Delta.y;	
	FTYPE NoveY=0.0f;
	if (InRealSpacePosOfP1+DataProMody[CurrentMode].Delta.y!=0) NoveY=p2.y/(InRealSpacePosOfP1+DataProMody[CurrentMode].Delta.y);	
	DataProMody[CurrentMode].Inited=true;

	if (!KeepAspect)
	{
		if (NoveX>0) DataProMody[CurrentMode].Scale.x=NoveX;
		if (NoveY>0) DataProMody[CurrentMode].Scale.y=NoveY;
	}
	else
	{
		if (NoveX<NoveY) NoveX=NoveY;
		if (NoveX>0) 
		{
			DataProMody[CurrentMode].Scale.x=NoveX;
			DataProMody[CurrentMode].Scale.y=NoveX;
		}

	}
}

GPSPoint DataToDisplayManager::GetPosition(POINTf &p)
{
	POINTf TmpVal;
	TmpVal.x=p.x;
	TmpVal.y=p.y;
	TmpVal.x/=DataProMody[CurrentMode].Scale.x;
	TmpVal.y/=DataProMody[CurrentMode].Scale.y;
	TmpVal.x-=DataProMody[CurrentMode].Delta.x;
	TmpVal.y-=DataProMody[CurrentMode].Delta.y;
	CDCDisplayInterface TmpCDC(CurrentMode,DataProMody[CurrentMode],RectangleOnScreen);
	return TmpCDC.SetSlozky(TmpVal);
}


listoflistoffilters &DataToDisplayManager::GetDataStorage(void)
{
	return DataToView;
}