#include "dataspace.h"
#include "ListOfFiltersDialog.h"
#include "DataToDisplayManager.h"
#include "filters\confinterface.h"
#include "DataDisplay.h"
#include "MainFrm.h"
#include "FilterSelectDialog.h"
#include "ListOfTracks.h"
#include "ConfigureTrack.h"
#include "MatchingConfig.h"
#include "TrackSelectDialog.h"



#include "..\shared\filters_processing\filter_load.h"

DataSpace GlobalDataSpace;

#include "filters\configurablefilter_clipstart.h"
#include "filters\configurablefilter_plovprumer.h"
#include "filters\configurablefilter_removenear.h"
#include "filters\configurablefilter_smothen.h"
#include "filters\configurablefilter_subdivide.h"
#include "filters\configurablefilter_deformmatch.h"
#include "filters\configurablefilter_defaultmerge.h"
#include "filters\configurablefilter_removestairs.h"
#include "filters\configurablefilter_save.h"

string IdAStringToString(int Id,string &s)
{
	char TmpBuff[1000];
	sprintf(TmpBuff,"%d:%s",Id,s.c_str());
	return TmpBuff;
}

int GetIdFromString(string &s)
{
	int id=s.find(':');
	return atoi(s.substr(0,id).c_str());
}

list<int> SelectTrack(bool Single) //zobrazi dialog pro vyber tracku a vrati cisla vybranych
{
	list<int> tmp;
	CTrackSelectDialog FSD;
	FSD.m_SingleSelection=Single;
	if (FSD.DoModal()==IDOK)
	{
		POSITION Pos=FSD.m_SelectedStrings.GetHeadPosition();
		while (Pos!=NULL)
			tmp.push_back(GetIdFromString(string((LPCSTR)FSD.m_SelectedStrings.GetNext(Pos))));
	}

	return tmp;
}


DataSpace::DataSpace(void)
{
	MatchesInvalidated=true;
	TracksInvalidated=true;
	FiltersInvaluidated=true;

	FreeTrackId=1;
	MatchingTrackId=1;
	FiltersDialog=NULL;
	FiltersDialogForMerge=NULL;
	TracksDialog=NULL;
	MatchingDialog=NULL;
	ActiveDataToDisplayManager=new DataToDisplayManager;

	UsedFilters.clear();
	UsedFiltersForMerge.clear();
	
	PossibleFilters.push_back(new ConfRemoveStairsFilter(4));
	PossibleFilters.push_back(new ConfSubdivideFilter(10));
	PossibleFilters.push_back(new ConfClipStartFilter(10,5,4,50));
	PossibleFilters.push_back(new ConfPlavouciPrumerFilter(10));
	PossibleFilters.push_back(new ConfRemoveNearFilter(10));
	PossibleFilters.push_back(new ConfSmothenFilter(10));	
	PossibleFilters.push_back(new ConfSaveGPSSegmentFilter("",true));
	PossibleFilters.push_back(new ConfDeformationMatchingFilter(40.0f,70.0f,200.0f,0.1f,10.0f,0.8f,600));
	PossibleFilters.push_back(new ConfDefaultMergeFilter(0,7,6,0,7,6));	
	
}

void DataSpace::Finalyze(void)
{
	Reset();
	while (PossibleFilters.size())
	{
		delete PossibleFilters.front();
		PossibleFilters.pop_front();
	}
	if (MatchingDialog) 
	{
		MatchingDialog->DestroyWindow();
		delete MatchingDialog;
	}
	MatchingDialog=NULL;
	if (FiltersDialogForMerge) 
	{
		FiltersDialogForMerge->DestroyWindow();
		delete FiltersDialogForMerge;
	}
	FiltersDialogForMerge=NULL;
	if (FiltersDialog) 
	{
		FiltersDialog->DestroyWindow();
		delete FiltersDialog;
	}
	FiltersDialog=NULL;
	if (TracksDialog) 
	{
		TracksDialog->DestroyWindow();
		delete TracksDialog;
	}
	TracksDialog=NULL;
	if (ActiveDataToDisplayManager) delete ActiveDataToDisplayManager;
	ActiveDataToDisplayManager=NULL;
}
	
DataSpace::~DataSpace(void)
{	
	Finalyze();
}

SingleTrackInfo *DataSpace::GetTrackById(int Id)
{
	listoftracks::iterator it;
	for (it=Tracks.begin();it!=Tracks.end();it++)
		if (it->TrackId==Id) return &(*it);
	return NULL;
}

SingleMatchingTrack *DataSpace::GetMatchById(int Id)
{
	listofmatchingtracks::iterator it;
	for (it=MatchingTracks.begin();it!=MatchingTracks.end();it++)
		if (it->Id==Id) return &(*it);
	return NULL;

}

struct DOPFPair
{
	RefCounterHolder<DisplayingOfProcessingFiltersListGPSPoint > MatchingGraph;
	RefCounterHolder<DisplayingOfProcessingFiltersListGPSPoint > MergingGraph;
};
typedef map<int,DOPFPair> MapDOPF;

void DataSpace::UpdateDialogs(bool NoDataRedo)
{
	LVITEM Itm;
	memset(&Itm,0,sizeof(Itm));
	Itm.mask=LVIF_TEXT|LVIF_PARAM|LVIF_DI_SETITEM ;


	listofmatchingtracks::iterator mit;
	listoftracks::iterator tit;
	ProcessingFiltersList<GPSPoint>::iterator it;

	if (!MatchingDialog)
	{
		MatchingDialog=new CMatchingConfig;
		MatchingDialog->Create(CMatchingConfig::IDD,NULL);
		MatchingDialog->SetInfoForMatch(-1);	
		MatchesInvalidated=true;
	}	

	if (MatchesInvalidated)
	{
		int Id=MatchingDialog->GetSelectedTrackId();
		MatchingDialog->SetInfoForMatch(-1);	
		MatchingDialog->m_Matches.DeleteAllItems();		
		for (mit=MatchingTracks.begin();mit!=MatchingTracks.end();mit++)
		{
			string s=IdAStringToString(mit->Id,mit->Name);
			Itm.pszText=(char *)s.c_str();		
			MatchingDialog->m_Matches.InsertItem(&Itm);		
		}	
		MatchingDialog->SetInfoForMatch(Id);			
		MatchesInvalidated=false;
	}
	MatchingDialog->ShowWindow(SW_SHOW);


	if (!TracksDialog)
	{
		TracksDialog=new CListOfTracks ;
		TracksDialog->Create(CListOfTracks::IDD,NULL);
		TracksInvalidated=true;
	}
	if (TracksInvalidated)
	{
		TracksDialog->m_List.DeleteAllItems();		
		for (tit=Tracks.begin();tit!=Tracks.end();tit++,Itm.iItem++)
		{
			string s=IdAStringToString(tit->TrackId,tit->FileName);
			Itm.pszText=(char *)s.c_str();
			Itm.lParam=Itm.iItem;
			TracksDialog->m_List.InsertItem(&Itm);
		}	
		TracksDialog->UpdateButtons();
		TracksInvalidated=false;
	}	
	TracksDialog->ShowWindow(SW_SHOW);


	if (!FiltersDialog)
	{
		FiltersDialog=new CListOfFiltersDialog;
		FiltersDialog->Create(CListOfFiltersDialog::IDD,NULL);		
		FiltersDialog->SetWindowText("Aktivn filtry pro hledn odpovdajcch si st");
		FiltersDialog->m_FiltersListIndex=0;
		FiltersInvaluidated=true;
	}
	if (!FiltersDialogForMerge)
	{
		FiltersDialogForMerge=new CListOfFiltersDialog;
		FiltersDialogForMerge->Create(CListOfFiltersDialog::IDD,NULL);	
		FiltersDialogForMerge->SetWindowText("Aktivn filtry pro sluovni");
		FiltersDialogForMerge->m_FiltersListIndex=1;
		FiltersInvaluidated=true;
	}
	
	if (FiltersInvaluidated)
	{
		FiltersDialog->m_List.DeleteAllItems();
		Itm.iItem=0;		
		for (it=UsedFilters.begin();it!=UsedFilters.end();it++,Itm.iItem++)
		{
			ConfigInterface *CINT=dynamic_cast<ConfigInterface *>((*it).GetValue());
			ASSERT(CINT);
			Itm.pszText=(char *)CINT->GetName();
			Itm.lParam=Itm.iItem;
			FiltersDialog->m_List.InsertItem(&Itm);
		}			 	
		FiltersDialog->UpdateButtons();

		FiltersDialogForMerge->m_List.DeleteAllItems();
		Itm.iItem=0;		
		for (it=UsedFiltersForMerge.begin();it!=UsedFiltersForMerge.end();it++,Itm.iItem++)
		{
			ConfigInterface *CINT=dynamic_cast<ConfigInterface *>((*it).GetValue());
			ASSERT(CINT);
			Itm.pszText=(char *)CINT->GetName();
			Itm.lParam=Itm.iItem;
			FiltersDialogForMerge->m_List.InsertItem(&Itm);
		}			 	
		FiltersDialogForMerge->UpdateButtons();
		FiltersInvaluidated=false;
	}
	FiltersDialog->ShowWindow(SW_SHOW);
	FiltersDialogForMerge->ShowWindow(SW_SHOW);

	if (NoDataRedo) return; //nemame predelat datove struktury v hlavnim okne

	MapDOPF IdToDisplayList; //sem se ulozi mapovani ID->Cachovany graf abychom ho mohli pouzit pri tvorbe mapovani

	//reinicializujeme ukazatele na cache pro mazani u vsech filtru na prazdne seznamy
	ProcessingFiltersList<GPSPoint>::iterator pfl;
	for (pfl=UsedFilters.begin();pfl!=UsedFilters.end();pfl++)
	{
		ConfigInterface *CINT=dynamic_cast<ConfigInterface *>(pfl->GetValue());
		ASSERT(CINT);
		CINT->SetCachesList(new ListOfInvalidatableCaches); //nastavime prazdny list
	}
	for (pfl=UsedFiltersForMerge.begin();pfl!=UsedFiltersForMerge.end();pfl++)
	{
		ConfigInterface *CINT=dynamic_cast<ConfigInterface *>(pfl->GetValue());
		ASSERT(CINT);
		CINT->SetCachesList(new ListOfInvalidatableCaches); //nastavime prazdny list
	}



	ActiveDataToDisplayManager->ClearListOfLists();
	for (tit=Tracks.begin();tit!=Tracks.end();tit++)
	{		
			DOPFPair Tmp;

			//vytvorime list pro matchovani
			//novy list
			RefCounterHolder<ProcessingFiltersList<GPSPoint> > PList=new ProcessingFiltersList<GPSPoint>;
			//pridani nacitaciho filtru
			PList->push_back(new LoadGPSSegmentFilter(tit->FileName));			
			//pridani referenci na pracovni filtry
			
			for (pfl=UsedFilters.begin();pfl!=UsedFilters.end();pfl++)
				PList->push_back((*pfl));											
			Tmp.MatchingGraph=ActiveDataToDisplayManager->CreateList(PList);
			
			//pridame tento graf do seznamu veci invalidovanych u vsech filtru v grafu
			for (pfl=UsedFilters.begin();pfl!=UsedFilters.end();pfl++)
			{
				ConfigInterface *CINT=dynamic_cast<ConfigInterface *>(pfl->GetValue());
				(CINT->GetCachesList()->push_back(Tmp.MatchingGraph.GetValue()));
			}

			
			//vytvorime list pro merging
			//novy list
			RefCounterHolder<ProcessingFiltersList<GPSPoint> > PList2=new ProcessingFiltersList<GPSPoint>;
			//pridani nacitaciho filtru
			PList2->push_back(new LoadGPSSegmentFilter(tit->FileName));			
			//pridani referenci na pracovni filtry
			for (pfl=UsedFiltersForMerge.begin();pfl!=UsedFiltersForMerge.end();pfl++)
				PList2->push_back((*pfl));
			Tmp.MergingGraph=ActiveDataToDisplayManager->CreateList(PList2);

			//pridame tento graf do seznamu veci invalidovanych u vsech filtru v grafu
			for (pfl=UsedFiltersForMerge.begin();pfl!=UsedFiltersForMerge.end();pfl++)
			{
				ConfigInterface *CINT=dynamic_cast<ConfigInterface *>(pfl->GetValue());
				(CINT->GetCachesList()->push_back(Tmp.MergingGraph.GetValue()));
			}

			
			ActiveDataToDisplayManager->AddList(Tmp.MatchingGraph,tit->Color);
			IdToDisplayList.insert(MapDOPF::value_type(tit->TrackId,Tmp));
	}	


	listofmatchingfilters &MlstLst=ActiveDataToDisplayManager->GetMatchesList();
	MlstLst.clear();	
	listofmatchingtracks::iterator mtit;
	for (mtit=MatchingTracks.begin();mtit!=MatchingTracks.end();mtit++)
	{
		if (mtit->MatchFilter)
		{
			SingleMatchingItem Tmp;
			Tmp.MatchFilter=mtit->MatchFilter;
			Tmp.MergeFilter=mtit->MergeFilter;
			

			ConfigInterface *CMatch=dynamic_cast<ConfigInterface *>(Tmp.MatchFilter.GetValue());
			ConfigInterface *CMerge=dynamic_cast<ConfigInterface *>(Tmp.MergeFilter.GetValue());
			
			//pridame CachedMerge do seznamu vsech filtru kterych se tyka
			if (CMerge) 
			{
				CMerge->SetCachesList(new ListOfInvalidatableCaches);
				CMerge->GetCachesList()->push_back(Tmp.CachedMerge.GetValue());
			}
					
			if (CMatch) 
			{
				CMatch->SetCachesList(new ListOfInvalidatableCaches);
				CMatch->GetCachesList()->push_back(Tmp.CachedMerge.GetValue()); //pokud se zmeni match tak se zmeni i merge
			}

			for (pfl=UsedFilters.begin();pfl!=UsedFilters.end();pfl++)
			{
				ConfigInterface *CINT=dynamic_cast<ConfigInterface *>(pfl->GetValue()); //pokud se zmeni matchovaci prvek tak se zmeni i merge
				(CINT->GetCachesList()->push_back(Tmp.CachedMerge.GetValue()));
			}

			for (pfl=UsedFiltersForMerge.begin();pfl!=UsedFiltersForMerge.end();pfl++) 
			{
				ConfigInterface *CINT=dynamic_cast<ConfigInterface *>(pfl->GetValue()); //pokud se zmeni mergovany prvek tak se zmeni i merge
				(CINT->GetCachesList()->push_back(Tmp.CachedMerge.GetValue()));
			}



			Tmp.Color=mtit->Color;
			Tmp.MergeColor=mtit->MergeColor;
			MapDOPF::iterator mdit=IdToDisplayList.find(mtit->PrimaryTrack);
			if (mdit!=IdToDisplayList.end())
			{
				Tmp.PrimaryTrack=mdit->second.MatchingGraph;
				Tmp.PrimaryTrackMergeGen=mdit->second.MergingGraph; //upravit
				list<int>::iterator iit;
				for (iit=mtit->SecondaryTracks.begin();iit!=mtit->SecondaryTracks.end();iit++)
				{
					MapDOPF::iterator mdit=IdToDisplayList.find(*iit);
					if (mdit!=IdToDisplayList.end())
					{
						SingleMatch SMatch;
						SMatch.CachedResult->Invalidate();
						SMatch.SecondaryTrackGraph=mdit->second.MatchingGraph;
						SMatch.SecondaryTrackGraphMergeGen=mdit->second.MergingGraph;
						Tmp.SecondaryTracks.push_back(SMatch);

						//doplnime seznamy pro mazani pri zmene
						if (CMatch) CMatch->GetCachesList()->push_back(SMatch.CachedResult.GetValue());
						for (pfl=UsedFilters.begin();pfl!=UsedFilters.end();pfl++)
						{
							ConfigInterface *CINT=dynamic_cast<ConfigInterface *>(pfl->GetValue()); //pokud se zmeni matchovaci prvek tak se zmeni i vysledny match
							(CINT->GetCachesList()->push_back(SMatch.CachedResult.GetValue()));
						}						
					}
				} 			
			}
			if (Tmp.SecondaryTracks.size())
				MlstLst.push_back(Tmp);

		}
	}

}

extern CDataDisplayApp theApp;

void DataSpace::ShowConfigurationDialog(int ListIndex,unsigned long ItemIndex)
{
	ProcessingFiltersList<GPSPoint> *FList=&UsedFilters;
	if (ListIndex==1) FList=&UsedFiltersForMerge;

	ProcessingFiltersList<GPSPoint>::iterator it=FList->begin();
	for (int q=0;q<ItemIndex && it!=FList->end();q++,it++);
	if (it!=FList->end())
	{
		ConfigInterface *CINT=dynamic_cast<ConfigInterface *>((*it).GetValue());
		CMainFrame *Frm=dynamic_cast<CMainFrame *>(theApp.m_pMainWnd);
		CINT->ShowConfig(Frm);
	}
}

typedef const char *pconstchar;

ConfigInterface *DataSpace::PickFilter(int FilterType)
{
	int SelIndex=-1;
	CFilterSelectDialog FSD;
	
	//alokujeme pole pro seznam jmen pouzitelnych filtru 
	const char **FilterNames=new pconstchar[PossibleFilters.size()+1]; //+1 za ukoncovaci NULL
	FSD.m_FilterNames=FilterNames;

	memset(FilterNames,0,sizeof(pconstchar)*PossibleFilters.size()+1); //vynulujeme, na konci bude jedna nula navic 
	listofpossiblefilters::iterator it;
	for (it=PossibleFilters.begin();it!=PossibleFilters.end();it++) 
	{
		if ((*it)->GetFilterClass()==FilterType)
			*(FilterNames++)=(*it)->GetName();	 //zapiseme jmena do seznamu
	}
	if (FSD.DoModal()==IDOK) SelIndex=FSD.m_SelectedFilter;	 //spustime dialog
	delete[] (FSD.m_FilterNames); //zrusime uz nepotrebny seznam

	if (SelIndex!=-1)
	{
		listofpossiblefilters::iterator it=PossibleFilters.begin();
		for (int q=0;q<SelIndex || (*it)->GetFilterClass()!=FilterType;it++)
			if ((*it)->GetFilterClass()==FilterType) q++;
		return (*it);
	}
	return NULL;

}

void DataSpace::AddItem(int ListIndex)
{
	ProcessingFiltersList<GPSPoint> *FList=&UsedFilters;
	if (ListIndex==1) FList=&UsedFiltersForMerge;

	ConfigInterface *CI=PickFilter(FILTERCLASS_PROCESSING);
	if (CI)
	{
		FList->push_back(CI->CreateNewProcessingFilterInstance());
		InvalidateFilters();
		UpdateDialogs();
		UpdateMainWindow();
	}

}

void DataSpace::MoveItemDown(int ListIndex,unsigned long ItemId)
{
	ProcessingFiltersList<GPSPoint> *FList=&UsedFilters;
	if (ListIndex==1) FList=&UsedFiltersForMerge;

	if (ItemId>=UsedFilters.size()-1) return;

	ProcessingFiltersList<GPSPoint>::iterator pfl;
	int c=0;
	for (pfl=FList->begin();pfl!=FList->end();pfl++,c++)
	{
		if (c==ItemId)
		{
			ProcessingFiltersList<GPSPoint>::iterator pfl2=pfl;
			pfl2++; //presun na dalsi prvek
			//prohodime prvky
			RefCounterHolder<ProcessingFilter<GPSPoint> > Tmp=*pfl;
			*pfl=*pfl2;
			*pfl2=Tmp;
			break;
		}
	}

	InvalidateFilters();
	UpdateDialogs();
	UpdateMainWindow();
}

void DataSpace::MoveItemUp(int ListIndex,unsigned long ItemId)
{
	ProcessingFiltersList<GPSPoint> *FList=&UsedFilters;
	if (ListIndex==1) FList=&UsedFiltersForMerge;

	if (ItemId==0) return;
	ProcessingFiltersList<GPSPoint>::iterator pfl;
	int c=0;
	for (pfl=FList->begin();pfl!=FList->end();pfl++,c++)
	{
		if (c==ItemId-1)
		{
			ProcessingFiltersList<GPSPoint>::iterator pfl2=pfl;
			pfl2++; //presun na dalsi prvek

			//prohodime prvky
			RefCounterHolder<ProcessingFilter<GPSPoint> > Tmp=*pfl;
			*pfl=*pfl2;
			*pfl2=Tmp;
			break;
		}
	}

	InvalidateFilters();
	UpdateDialogs();
	UpdateMainWindow();
}

void DataSpace::DeleteItem(int ListIndex,unsigned long ItemId)
{
	ProcessingFiltersList<GPSPoint> *FList=&UsedFilters;
	if (ListIndex==1) FList=&UsedFiltersForMerge;

	if (ItemId>=FList->size()) return;

	ProcessingFiltersList<GPSPoint>::iterator pfl;
	int c=0;
	for (pfl=FList->begin();pfl!=FList->end();pfl++,c++)
		if (c==ItemId) 
		{
			ConfigInterface *CINT=dynamic_cast<ConfigInterface *>(pfl->GetValue());
			CINT->GetCachesList()->clear(); //zrusime caches
			FList->erase(pfl);
			break;
		}

	InvalidateFilters();
	UpdateDialogs();
	UpdateMainWindow();
}



char FileDialogFileNames[100000];
void DataSpace::AddTrack(void)
{
	char FilterBuffer[1000];
	strcpy(FilterBuffer,"Segment files (*.seg)|*.seg|All Files (*.*)|*.*||");

	SingleTrackInfo TInfo;	
	CFileDialog FDLG(TRUE,"seg",NULL,OFN_ALLOWMULTISELECT|OFN_EXPLORER|OFN_FILEMUSTEXIST|OFN_PATHMUSTEXIST,FilterBuffer,theApp.m_pMainWnd);
	OPENFILENAME &OFNData=FDLG.m_ofn;
	OFNData.Flags|=OFN_ALLOWMULTISELECT|OFN_EXPLORER|OFN_FILEMUSTEXIST|OFN_PATHMUSTEXIST;
	OFNData.lpstrFile=FileDialogFileNames;
	OFNData.nMaxFile=sizeof(FileDialogFileNames)-1;
	if (FDLG.DoModal()==IDOK)
	{
		if (OFNData.nFileOffset>strlen(FileDialogFileNames))
		{
			//vybrano vice souboru
			char *Current=FileDialogFileNames;
			if (*Current)
			{
				string DirName=Current;
				if (DirName.length())
					if (DirName.c_str()[DirName.length()-1]!='\\') DirName+="\\";
				//na zacatku je adresar, preskocime ho
				Current+=strlen(Current)+1;
				while (*Current)
				{
					TInfo.FileName=DirName+string(Current);
					TInfo.Color=0xffffff;
					TInfo.TrackId=FreeTrackId++;
					Tracks.push_back(TInfo);
					Current+=strlen(Current)+1;
				}
			}
		}
		else
		{
			//vybran jeden soubor
			TInfo.FileName=string(FileDialogFileNames);
			TInfo.Color=0xffffff;
			TInfo.TrackId=FreeTrackId++;
			Tracks.push_back(TInfo);
		}
		InvalidateTracks();
		UpdateDialogs();
		UpdateMainWindow();
	}

}

typedef listoftracks::iterator LOTIT;
void DataSpace::DeleteTracks(int Count,unsigned long *TracksIds)
{
	if (!Count) return;
	
	//DULEZITE:nasledujici kod pouziva toho ze erase v listu nezneplatnuje iteratory ktere neukazuji na mazany prvek

	listoftracks::iterator *Its=new LOTIT[Count];
	for (int j=0;j<Count;j++) Its[j]=Tracks.end();
	
	//ziskame seznam iteratoru ktere je potreba vyhodit
	listoftracks::iterator it;
	for (it=Tracks.begin();it!=Tracks.end();it++)
		for (int j=0;j<Count;j++)
			if (it->TrackId==TracksIds[j]) 
			{
				Its[j]=it; //tento prvek je v seznamu mazanych, poznamename si jeho iterator pro pozdejsi smazani
				break;
			}


	//vyhodime ze vsech match bloku
	listofmatchingtracks::iterator itx;
	for (itx=MatchingTracks.begin();itx!=MatchingTracks.end();itx++)
	{
		for (int j=0;j<Count;j++)
			if (itx->PrimaryTrack==TracksIds[j])
			{
				itx->PrimaryTrack=-1;
				InvalidateMatches();
			}

		list<int>::iterator itx2; 
		for (itx2=itx->SecondaryTracks.begin();itx2!=itx->SecondaryTracks.end();)
		{
			bool Erase=false;
			for (int j=0;j<Count;j++)
				if (*itx2==TracksIds[j])
					Erase=true;

			if (Erase)
			{
				itx2=itx->SecondaryTracks.erase(itx2);
				InvalidateMatches();
			}
			else
				itx2++;
		}
	}

	//vyhodime dane iteratory
	for (int i=0;i<Count;i++)
		if (Its[i]!=Tracks.end())
			Tracks.erase(Its[i]);
	delete[] Its;


	InvalidateTracks();
	UpdateDialogs();
	UpdateMainWindow();
}


void DataSpace::ConfigureTrack(unsigned long ItemId)
{
	listoftracks::iterator tit;
	for (tit=Tracks.begin();tit!=Tracks.end();tit++)
	{
		if (tit->TrackId==ItemId)
		{
			CConfigureTrack CTRK;
			CTRK.m_Color=tit->Color;
			if (CTRK.DoModal()==IDOK)
				tit->Color=CTRK.m_Color;
		}
	}		

	InvalidateTracks();
	UpdateDialogs();
	UpdateMainWindow();
}

void DataSpace::UpdateMainWindow(void)
{
	CMainFrame *Frm=dynamic_cast<CMainFrame *>(theApp.m_pMainWnd);
	Frm->RedoGraph();
}

GPSPointBuffer::Holder DataSpace::GenerateResultCurveForSelectedMatch(SingleMatchingTrack *t)
{
	
	map<int,GPSPointBuffer::Holder> PreprocessingForMatching;
	map<int,GPSPointBuffer::Holder> PreprocessingForMerging;


	SingleTrackInfo *TInfo=GetTrackById(t->PrimaryTrack);
	if (!TInfo) return NULL; //nepovedlo se pro primarni->nic nedelame
	if (!t->MatchFilter) return NULL;
	if (!t->MergeFilter) return NULL;

	RefCounterHolder<ProcessingFiltersList<GPSPoint> > MatchFilters=new ProcessingFiltersList<GPSPoint>;
	MatchFilters->push_back(new LoadGPSSegmentFilter(TInfo->FileName));	
	ProcessingFiltersList<GPSPoint> ::iterator pfl;
	for (pfl=UsedFilters.begin();pfl!=UsedFilters.end();pfl++)
			MatchFilters->push_back((*pfl));

	GPSPointBuffer::Holder TmpIn=new GPSPointBuffer;
	GPSPointBuffer::Holder TmpOut;
	MatchFilters->ExecuteList(TmpIn,TmpOut);
	PreprocessingForMatching[t->PrimaryTrack]=TmpOut;



		
	RefCounterHolder<ProcessingFiltersList<GPSPoint> > MergeFilters=new ProcessingFiltersList<GPSPoint>;
	MergeFilters->push_back(new LoadGPSSegmentFilter(TInfo->FileName));	
	for (pfl=UsedFiltersForMerge.begin();pfl!=UsedFiltersForMerge.end();pfl++)
			MergeFilters->push_back((*pfl));

	TmpIn=new GPSPointBuffer;
	TmpOut=NULL;
	MergeFilters->ExecuteList(TmpIn,TmpOut);
	PreprocessingForMerging[t->PrimaryTrack]=TmpOut;


	RefCounterHolder<Buffer<MatchPair<GPSPoint> > > MatchingResults=new Buffer<MatchPair<GPSPoint> >;
	list<int>::iterator it;
	for (it=t->SecondaryTracks.begin();it!=t->SecondaryTracks.end();it++)
	{
		SingleTrackInfo *TInfo=GetTrackById(*it);
		if (TInfo) 
		{
			//spocitame krivky pro ostatni trasy
			RefCounterHolder<ProcessingFiltersList<GPSPoint> > MatchFilters=new ProcessingFiltersList<GPSPoint>;
			MatchFilters->push_back(new LoadGPSSegmentFilter(TInfo->FileName));	
			for (pfl=UsedFilters.begin();pfl!=UsedFilters.end();pfl++)
				MatchFilters->push_back((*pfl));

			GPSPointBuffer::Holder TmpIn=new GPSPointBuffer;
			GPSPointBuffer::Holder TmpOut;
			MatchFilters->ExecuteList(TmpIn,TmpOut);
			PreprocessingForMatching[*it]=TmpOut;


		
			RefCounterHolder<ProcessingFiltersList<GPSPoint> > MergeFilters=new ProcessingFiltersList<GPSPoint>;
			MergeFilters->push_back(new LoadGPSSegmentFilter(TInfo->FileName));	
			for (pfl=UsedFiltersForMerge.begin();pfl!=UsedFiltersForMerge.end();pfl++)
				MergeFilters->push_back((*pfl));

			TmpIn=new GPSPointBuffer;
			TmpOut=NULL;
			MergeFilters->ExecuteList(TmpIn,TmpOut);
			PreprocessingForMerging[*it]=TmpOut;


			//spocitame match
			Buffer<CrossInfo<GPSPoint> >::Holder MatchResult;			
			t->MatchFilter->MatchCurves(PreprocessingForMatching[t->PrimaryTrack],PreprocessingForMatching[*it],MatchResult);

			//vytvorime vysledny pair z krivky pro mergovani a vysledku matchovani s krivnou pro matchovani
			MatchPair<GPSPoint> TmpMPair;
			TmpMPair.Curve=PreprocessingForMerging[*it];
			TmpMPair.MatchData=MatchResult;
			MatchingResults->Data.push_back(TmpMPair);
		}

	}

	//zavolame mergovani
	GPSPointBuffer::Holder Result;
	t->MergeFilter->GenerateResult(PreprocessingForMerging[t->PrimaryTrack],MatchingResults,Result);
	return Result;
	
}

void operator<<=(Archive &archiv,SingleTrackInfo &t)
{
	archiv<<=t.TrackId;
	archiv<<=t.FileName;
	archiv<<=t.Color;
}

void operator<<=(Archive &archiv,SingleMatchingTrack &t)
{
	archiv<<=t.Id;
	archiv<<=t.Name;
	archiv<<=t.Color;
	archiv<<=t.MergeColor;
	archiv<<=t.PrimaryTrack;
	archiv<<=t.SecondaryTracks;


	Archivable *tmp=dynamic_cast<Archivable *>(t.MatchFilter.GetValue());
	archiv<<=tmp;
	t.MatchFilter=dynamic_cast<MatchingFilter<GPSPoint> *>(tmp);

	tmp=dynamic_cast<Archivable *>(t.MergeFilter.GetValue());
	archiv<<=tmp;
	t.MergeFilter=dynamic_cast<MergingFilter<GPSPoint> *>(tmp);
}

#ifdef RegisterClass
#undef RegisterClass
#endif

void DataSpace::Save(string FileName)
{
   FILEArchiveDataSource DSource(FileName.c_str(),false);
   if (!DSource.IsValid())
   {
	   string TmpFile=string("Vytvoen souboru selhalo");
	   MessageBox(NULL,TmpFile.c_str(),"Chyba",MB_OK);
	   return;
   }
   RefCounterHolder<Archive> Archiv=Archive::CreateArchive();


   //zaregistrujeme filtry ktere se ukladaji
   listofpossiblefilters::iterator itf;
   for (itf=PossibleFilters.begin();itf!=PossibleFilters.end();itf++)
   { 
	   ConfigInterface *Ci=*itf;
	   Archiv->RegisterClass(dynamic_cast<Archivable *>(Ci));
   }

   Archiv->InitArchive(&DSource,false);




   //provedeme vlastni ulozeni
   (*Archiv)<<=FreeTrackId;
   (*Archiv)<<=MatchingTrackId;

   ProcessingFiltersList<GPSPoint>::iterator it;
   int Cnt=UsedFilters.size();
   (*Archiv)<<=Cnt;
   for (it=UsedFilters.begin();it!=UsedFilters.end();it++)
   {
		Archivable *tmp=dynamic_cast<Archivable *>(it->GetValue());
		(*Archiv)<<=tmp;
   }

   Cnt=UsedFiltersForMerge.size();
   (*Archiv)<<=Cnt;
   for (it=UsedFiltersForMerge.begin();it!=UsedFiltersForMerge.end();it++)
   {
		Archivable *tmp=dynamic_cast<Archivable *>(it->GetValue());
		(*Archiv)<<=tmp;
   }
   (*Archiv)<<=MatchingTracks;
   (*Archiv)<<=Tracks;
   Archiv->DoneArchive();   
}

void DataSpace::Load(string FileName)
{
   Reset();
   FILEArchiveDataSource DSource(FileName.c_str(),true);
   if (!DSource.IsValid())
   {
	   string TmpFile=string("Oteven souboru selhalo");
	   MessageBox(NULL,TmpFile.c_str(),"Error",MB_OK);
	   return;
   }

   RefCounterHolder<Archive> Archiv=Archive::CreateArchive();

	//zaregistrujeme filtry ktere se nacitaji
   listofpossiblefilters::iterator itf;
   for (itf=PossibleFilters.begin();itf!=PossibleFilters.end();itf++)
   { 
	   ConfigInterface *Ci=*itf;
	   Archiv->RegisterClass(dynamic_cast<Archivable *>(Ci));
   }

   Archiv->InitArchive(&DSource,true);

   


   //provedeme nacteni

   (*Archiv)<<=FreeTrackId;
   (*Archiv)<<=MatchingTrackId;

   int Cnt,x;
   (*Archiv)<<=Cnt;
   for (x=0;x<Cnt;x++)
   {
	   Archivable *tmp=NULL;
	   (*Archiv)<<=tmp;
	   UsedFilters.push_back(dynamic_cast<ProcessingFilter<GPSPoint> *>(tmp));
   }

   (*Archiv)<<=Cnt;
   for (x=0;x<Cnt;x++)
   {
	   Archivable *tmp=NULL;
	   (*Archiv)<<=tmp;
	   UsedFiltersForMerge.push_back(dynamic_cast<ProcessingFilter<GPSPoint> *>(tmp));
   }
   (*Archiv)<<=MatchingTracks;
   (*Archiv)<<=Tracks;
    Archiv->DoneArchive();  
	UpdateDialogs(false);
	UpdateMainWindow();

}
void DataSpace::Reset(void)
{
	FreeTrackId=1;
	MatchingTrackId=1;
	MatchesInvalidated=true;
	TracksInvalidated=true;
	FiltersInvaluidated=true;


	while (UsedFilters.size())
	{
		ConfigInterface *CINT=dynamic_cast<ConfigInterface *>(UsedFilters.front().GetValue());
		if (CINT) CINT->GetCachesList()->clear(); //zrusime caches
		UsedFilters.pop_front();
	}
	while (UsedFiltersForMerge.size())
	{
		ConfigInterface *CINT=dynamic_cast<ConfigInterface *>(UsedFiltersForMerge.front().GetValue());
		if (CINT) CINT->GetCachesList()->clear(); //zrusime caches
		UsedFiltersForMerge.pop_front();
	}

	while (MatchingTracks.size())
	{
		ConfigInterface *CINT=dynamic_cast<ConfigInterface *>(MatchingTracks.front().MatchFilter.GetValue());
		if (CINT) CINT->GetCachesList()->clear(); //zrusime caches
		CINT=dynamic_cast<ConfigInterface *>(MatchingTracks.front().MergeFilter.GetValue());
		if (CINT) CINT->GetCachesList()->clear(); //zrusime caches

		MatchingTracks.pop_front();
	}
		
	Tracks.clear();
}
