#ifndef DATASPACE_H
#define DATASPACE_H


#include "..\shared\gpspoint.h"
#include "..\shared\display.h"
#include "..\shared\filterlist.h"
#include "stdafx.h"
#include "filters\confinterface.h"


class ConfigInterface;
class CListOfFiltersDialog;
class DataToDisplayManager;
class CListOfTracks;
class CMatchingConfig;


/////////////////////// datove struktury pouzivane v DataToDisplayManager
typedef RefCounterHolder<GPSPointBuffer> &GPSPBHolderRef;
class InvalidableGPSPointBufferHolder:public InvalidatableCache
{	
	RefCounterHolder<GPSPointBuffer> Data;
	public:
	operator GPSPBHolderRef(void) {return Data;}
	GPSPointBuffer *GetValue(void) {return Data;}
	virtual void Invalidate(void) {Data=NULL;}
	bool IsNULL(void) {return Data.GetValue()==NULL;}
	void SetTo(RefCounterHolder<GPSPointBuffer> D) {Data=D;}
};

typedef Buffer<CrossInfo<GPSPoint> >::Holder &CBHolderRef;
class InvalidableCBufferHolder:public InvalidatableCache
{	
	Buffer<CrossInfo<GPSPoint> >::Holder Data;
	public:
	operator CBHolderRef(void) {return Data;}
	Buffer<CrossInfo<GPSPoint> > *GetValue(void) {return Data.GetValue();}
	virtual void Invalidate(void) {Data=NULL;}	
	bool IsNULL(void) {return Data.GetValue()==NULL;}
	void SetTo(Buffer<CrossInfo<GPSPoint> >::Holder D) {Data=D;}
};

typedef RefCounterHolder<DisplayingOfProcessingFiltersList<GPSPoint> > &DOPFLHolderRef;
class DisplayingOfProcessingFiltersListGPSPoint:public InvalidatableCache 
{
	RefCounterHolder<DisplayingOfProcessingFiltersList<GPSPoint> > Data;
	public:

	operator DOPFLHolderRef(void) {return Data;}
	DisplayingOfProcessingFiltersList<GPSPoint> *GetValue(void) {return Data.GetValue();}
	virtual void Invalidate(void) {Data->ClearCache();}		

	bool IsNULL(void) {return Data.GetValue()==NULL;}
	void SetTo(RefCounterHolder<DisplayingOfProcessingFiltersList<GPSPoint> > D) {Data=D;}	
};


typedef RefCounterHolder<DisplayingOfProcessingFiltersListGPSPoint > filterslist;
struct SingleListItem
{
	filterslist GraphToDraw;
	COLORREF DisplayColor;
};


struct SingleMatch
{
	filterslist SecondaryTrackGraph;
	filterslist SecondaryTrackGraphMergeGen;
	RefCounterHolder<InvalidableCBufferHolder> CachedResult;
	
	SingleMatch(void)
	{
		CachedResult=new InvalidableCBufferHolder;
	}
};

struct SingleMatchingItem
{
	filterslist PrimaryTrack;
	filterslist PrimaryTrackMergeGen;
	list<SingleMatch> SecondaryTracks;

	RefCounterHolder<MatchingFilter<GPSPoint> > MatchFilter; //pouzit na kazdou 	
	RefCounterHolder<MergingFilter<GPSPoint> > MergeFilter;
	RefCounterHolder<InvalidableGPSPointBufferHolder> CachedMerge;

	DWORD Color;
	DWORD MergeColor;

	SingleMatchingItem(void)
	{
		CachedMerge=new InvalidableGPSPointBufferHolder;
	}

	void Reset(void)
	{
		list<SingleMatch>::iterator it;
		for (it=SecondaryTracks.begin();it!=SecondaryTracks.end();it++) it->CachedResult->Invalidate();
	}

	void Recalculate(void)
	{
		Buffer<GPSPoint>::Holder PrimaryTrackBuffer=PrimaryTrack->GetValue()->GetCachedBuffer();			
		list<SingleMatch>::iterator it;
		for (it=SecondaryTracks.begin();it!=SecondaryTracks.end();it++)
			if (it->CachedResult->IsNULL())
			{
				Buffer<GPSPoint>::Holder SecondaryTrackBuffer=it->SecondaryTrackGraph->GetValue()->GetCachedBuffer();			
				MatchFilter->MatchCurves(PrimaryTrackBuffer,SecondaryTrackBuffer,*it->CachedResult);
			}
	}
};


typedef vector<SingleListItem> listoflistoffilters;
typedef vector<SingleMatchingItem> listofmatchingfilters;

//////////////////////datove struktury pro ulozeni informaci
struct SingleTrackInfo //infrmace o jedne nactene trase
{
	int TrackId;
	string FileName;
	COLORREF Color;
};

struct SingleMatchingTrack //informace o jednom sluovacm bloku (MatchFilter,PrimarniTrack a soubor Sekundarnich)
{
	int Id;
	string Name; //pro uzivatele	
	COLORREF Color;
	COLORREF MergeColor;
	RefCounterHolder<MatchingFilter<GPSPoint> > MatchFilter;
	RefCounterHolder<MergingFilter<GPSPoint> > MergeFilter;
	int PrimaryTrack;
	list<int> SecondaryTracks;

	SingleMatchingTrack(void)
	{
		Id=0;
		PrimaryTrack=-1;
		Color=0xffffff;
	}

};


typedef list<SingleTrackInfo> listoftracks;
typedef list<SingleMatchingTrack> listofmatchingtracks;
typedef list<ConfigInterface *> listofpossiblefilters;


//tato trida uklada spolecne informace o datech

class DataSpace
{
public:
	int FreeTrackId;
	int MatchingTrackId;

	listofpossiblefilters PossibleFilters; //filtry, ktere je mozno vybirat pro pridani
	listofmatchingtracks MatchingTracks;   //informace o vsechn matchovacich blocch
	ProcessingFiltersList<GPSPoint> UsedFilters; //seznam pouzivanych filtru
	ProcessingFiltersList<GPSPoint> UsedFiltersForMerge; //seznam pouzivanych filtru
	

	listoftracks Tracks; //seznam zadanych trati

	CListOfFiltersDialog *FiltersDialog; //dialog se seznamy filtru
	CListOfFiltersDialog *FiltersDialogForMerge; //dialog se seznamy filtru pro mergovani
	CListOfTracks *TracksDialog; //dialog se seznamy tracku
	CMatchingConfig *MatchingDialog; //dialog se seznamem matchu
	DataToDisplayManager *ActiveDataToDisplayManager;


	bool MatchesInvalidated; //pokud je true pak se pri dalsim UpdateDialogs updatne dialog pro konfiguraci slucovacich bloku
	bool TracksInvalidated; //pokud je true pak se pri dalsim UpdateDialogs updatne dialog se seznamem tras
	bool FiltersInvaluidated; //pokud je true pak se pri dalsim UpdateDialogs updatne dialog se seznamem filtru
	

	void Save(string FileName);
	void Load(string FileName);
	void Reset(void);

	void Finalyze(void); //provede uvolneni vsech objektu, volat pri ukonceni programu

	DataSpace(void);
	~DataSpace(void);
	
	void UpdateDialogs(bool NoDataRedo=false); //naplni dialogy podle dat, MUSI se volat pri kazde zmene datovych struktur 
	void UpdateMainWindow(void);
	void ShowConfigurationDialog(int ListIndex,unsigned long ItemId);

	//volano z dialogu pro manipulaci s filtry
	void AddItem(int ListIndex); //zobrazi dialog pro vyber filtru pro zadany list
	void MoveItemDown(int ListIndex,unsigned long ItemId);
	void MoveItemUp(int ListIndex,unsigned long ItemId);
	void DeleteItem(int ListIndex,unsigned long ItemId);

	//volano s dialogu pro manipulaci s soubory
	void AddTrack(void); //zobrazi dialog pro pridani tras
	void ConfigureTrack(unsigned long ItemIndex); //zobrazi konfiguracni dialog pro konfiguraci trasy se zadanym id
	void DeleteTracks(int Count,unsigned long *Tracks); //zrusi trasy se zadanymi Id


	void InvalidateMatches(void) {MatchesInvalidated=true;}
	void InvalidateTracks(void) {TracksInvalidated=true;}
	void InvalidateFilters(void) {FiltersInvaluidated=true;}

	SingleTrackInfo *GetTrackById(int Id); //vraci ukazatel na datovou strukturu pro prislusnou trasu podle Id trasy
	SingleMatchingTrack *GetMatchById(int Id); //vraci ukazatel na datovou strukturu pro prislusne matchovani, podle id matchovani

	ConfigInterface *PickFilter(int FilterType); //zobrazi dialog pro vyber filtru zadaneho typu, vraci interface ktery je mozny pouzit pro tvorbu nove instance

	GPSPointBuffer::Holder GenerateResultCurveForSelectedMatch(SingleMatchingTrack *); //spocita vyslednou krivku po matchovani a mergovani
};
string IdAStringToString(int Id,string &s);
int GetIdFromString(string &s);


list<int> SelectTrack(bool Single); //zobrazi dialog pro vyber tracku


extern DataSpace GlobalDataSpace;

#endif