#ifndef CONFIG_INTERFACE_H
#define CONFIG_INTERFACE_H

#include <stdlib.h>
#include "..\..\shared\gpspoint.h"
#include "..\..\shared\filter.h"
#include "..\..\shared\archivestl.h"


#define FILTERCLASS_PROCESSING 1
#define FILTERCLASS_MATCHING 2
#define FILTERCLASS_MERGING 3
class CDialog;
class CMainFrame;

class InvalidatableCache:public RefCounter
{
	public:
	virtual void Invalidate(void)=0;
};

class ListOfInvalidatableCaches:public list<RefCounterHolder<InvalidatableCache,true> >,public RefCounter
{
public:
	typedef RefCounterHolder<ListOfInvalidatableCaches> Holder;
	void InvalidateAll(void);
};

class ConfigInterface:public Archivable
{
	protected:
	CDialog *DialogPointer; //ukazatel na pointer otevreneho dialogu
	CMainFrame *MainFrame;  //ukazatel na hlavni okno aplikace

	ListOfInvalidatableCaches::Holder Caches;
	public:
	ConfigInterface(void) {DialogPointer=NULL;MainFrame=NULL;}
	virtual ~ConfigInterface(void);

	void SetCachesList(ListOfInvalidatableCaches::Holder CachesList); 
	ListOfInvalidatableCaches::Holder GetCachesList(void);
	
	void ShowConfig(CMainFrame *);
	void DialogUpdated(void);

	//nasledujici funkce predefinovava konkretni filtr

	virtual int GetFilterClass(void)=0;
	virtual ProcessingFilter<GPSPoint> *CreateNewProcessingFilterInstance(void) {return NULL;} //vytvori instanci noveho filtru
	virtual MatchingFilter<GPSPoint> *CreateNewMatchingFilterInstance(void) {return NULL;}
	virtual MergingFilter<GPSPoint> *CreateNewMergingFilterInstance(void) {return NULL;}
	virtual const char *GetName(void)=0; //vraci jmeno filtru
	virtual CDialog *CreateConfigDialog(void)=0; //vytvori objekt konfiguracniho dialogu
	virtual void SyncConfigDialog(CDialog *SyncWithDialog,bool ConfigureDialog)=0;
	//prenese data mezi dialogem a objektem pokud ConfigureDialog==true a jinak opacne
};


#ifdef GetClassName 
#undef GetClassName
#endif

#define CIDEFAULTMEMBERS(Name)\
	virtual int GetFilterClass(void) {return FILTERCLASS_PROCESSING;}\
	virtual const char *GetName(void) {return Name;}\
	virtual ProcessingFilter<GPSPoint> *CreateNewProcessingFilterInstance(void); \
	virtual CDialog *CreateConfigDialog(void); \
	virtual void SyncConfigDialog(CDialog *SyncWithDialog,bool ConfigureDialog);\
    virtual void DoOperation(Archive &archiv,int Version);\
	virtual Archivable *ConstructClassInstance(void) {return dynamic_cast<Archivable *>(CreateNewProcessingFilterInstance());}\
    virtual void DestructClassInstance(Archivable *I) {if (I) delete (CIFILTERNAME *)I;}\
    virtual bool IsSupportedForLoad(int Version) {return Version==1;}\
    virtual int GetSavedVersion(void) {return 1;}\
    virtual void Invalidate(void) {}\
	virtual const char *GetClassName(void) {return CIFILTERNAMESTR;}

#define CIDEFAULTMEMBERSMATCH(Name)\
	virtual int GetFilterClass(void) {return FILTERCLASS_MATCHING;}\
	virtual const char *GetName(void) {return Name;}\
	virtual MatchingFilter<GPSPoint> *CreateNewMatchingFilterInstance(void); \
	virtual CDialog *CreateConfigDialog(void); \
	virtual void SyncConfigDialog(CDialog *SyncWithDialog,bool ConfigureDialog); \
    virtual void DoOperation(Archive &archiv,int Version);\
    virtual Archivable *ConstructClassInstance(void) {return dynamic_cast<Archivable *>(CreateNewMatchingFilterInstance());}\
    virtual void DestructClassInstance(Archivable *I) {if (I) delete (CIFILTERNAME *)I;}\
    virtual bool IsSupportedForLoad(int Version) {return Version==1;}\
    virtual int GetSavedVersion(void) {return 1;}\
    virtual void Invalidate(void) {}\
	virtual const char *GetClassName(void) {return CIFILTERNAMESTR;}

#define CIDEFAULTMEMBERSMERGE(Name)\
	virtual int GetFilterClass(void) {return FILTERCLASS_MERGING;}\
	virtual const char *GetName(void) {return Name;}\
	virtual MergingFilter<GPSPoint> *CreateNewMergingFilterInstance(void);  \
	virtual CDialog *CreateConfigDialog(void); \
	virtual void SyncConfigDialog(CDialog *SyncWithDialog,bool ConfigureDialog);\
    virtual void DoOperation(Archive &archiv,int Version);\
    virtual Archivable *ConstructClassInstance(void) {return dynamic_cast<Archivable *>(CreateNewMergingFilterInstance());}\
    virtual void DestructClassInstance(Archivable *I) {if (I) delete (CIFILTERNAME *)I;}\
    virtual bool IsSupportedForLoad(int Version) {return Version==1;}\
    virtual int GetSavedVersion(void) {return 1;}\
    virtual void Invalidate(void) {}\
	virtual const char *GetClassName(void) {return CIFILTERNAMESTR;}

#endif