// DataExport.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "../shared/refcount.h"
#include "../shared/archivestl.h"
#include "../gpsparser/gpsfileparser.h"
#include "../shared/filters_processing/filter_load.h"

#define DWORD unsigned int 

void GenerateTextOutput(GPSPointBuffer::Holder Data,string FileName,FTYPE ScaleD,FTYPE ScaleS)
{
	//vygenerovani textoveho vystupu
	//co radka to jeden bod  delka,sirka, vyska
	FILE *f=fopen(FileName.c_str(),"wt");
	GPSPointBuffer::BufferIterator it;
	for (it=Data->Data.begin();it!=Data->Data.end();it++)
	{
		if (it!=Data->Data.begin()) fprintf(f,"\n");//aby vsechno bylo na samostatnych radkach
		VECT3D v=*it;
		fprintf(f,"%f,%f,%f",v.d*ScaleD,v.s*ScaleS,v.v);
	}	
	fclose(f);
}




void DrawLine(DWORD *Bitmap,DWORD SirkaBitmapy,DWORD VyskaBitmapy,FTYPE D1,FTYPE S1,FTYPE D2,FTYPE S2,DWORD Color)
{
	FTYPE DD=D2-D1;
	FTYPE DS=S2-S1;
	

	FTYPE Length=sqrt(DS*DS+DD*DD);
	DD/=Length;
	DS/=Length;
	

	FTYPE L=0;
	while (L<Length)
	{		
		DWORD OutputX=10+D1;
		DWORD OutputY=10+S1;

		if (OutputX>=0 && OutputY>=0 && OutputX<SirkaBitmapy && OutputY<VyskaBitmapy)
			Bitmap[OutputX+OutputY*SirkaBitmapy]=0xff000000|Color;		
		D1+=0.5*DD;
		S1+=0.5*DS;
		L+=0.5;
	}
}

void GenerateImageOutput(GPSPointBuffer::Holder Data,string FileName,FTYPE ScaleD,FTYPE ScaleS,DWORD Color,bool UseRect,FTYPE LeftTopD=0.0,FTYPE LeftTopS=0.0,FTYPE RightBottomD=0.0,FTYPE RightBottomS=0.0)
{
	//vygeneruje obrazek ve formatu tga
#pragma pack (push, imageheader)
#pragma pack (1)

	struct
	{
		unsigned char IdLength;
		unsigned char ColorMapType;
		unsigned char ImageType;
		unsigned short CMapStart;
		unsigned short CMapLength;
		unsigned char CMapDepth;
		unsigned short XOffset;
		unsigned short YOffSet;
		unsigned short Width;
		unsigned short Height;
		unsigned char PixelDepth;
		unsigned char ImageDescriptor;
	} Header;
#pragma pack (pop,imageheader)
	

	FTYPE MinS,MinD;
	FTYPE MaxS,MaxD;


	GPSPointBuffer::BufferIterator it=Data->Data.begin();
	VECT3D V=*it;
	MaxD=MinD=V.d;
	MaxS=MinS=V.s;	
	it++;
	for (;it!=Data->Data.end();it++)
	{
		VECT3D V=*it;
		if (MaxD<V.d) MaxD=V.d;
		if (MaxS<V.s) MaxS=V.s;		
		if (V.d<MinD) MinD=V.d;
		if (V.s<MinS) MinS=V.s;		
	}

	if (UseRect)
	{
		MinD=LeftTopD;
		MaxS=LeftTopS;

		MaxD=RightBottomD;
		MinS=RightBottomS;
	}


	FTYPE RozdilD=(MaxD-MinD)*ScaleD;
	FTYPE RozdilS=(MaxS-MinS)*ScaleS;

	
	DWORD SirkaBitmapy=20+RozdilD;
	DWORD VyskaBitmapy=20+RozdilS;

	DWORD *Bitmap=new DWORD[SirkaBitmapy*VyskaBitmapy];
	memset(Bitmap,0,sizeof(DWORD)*SirkaBitmapy*VyskaBitmapy);

	FTYPE LastS,LastD;

	for (it=Data->Data.begin();it!=Data->Data.end();it++)
	{
		VECT3D V=*it;
		FTYPE D=(V.d-MinD)*ScaleD;
		FTYPE S=(V.s-MinS)*ScaleS;		
		if (it!=Data->Data.begin())
			DrawLine(Bitmap,SirkaBitmapy,VyskaBitmapy,LastD,LastS,D,S,Color);
		else
			DrawLine(Bitmap,SirkaBitmapy,VyskaBitmapy,D,S,D,S,Color);
		LastD=D;
		LastS=S;		
	}



	//zapiseme na vystup
	FILE *f=fopen(FileName.c_str(),"wb");	
	memset(&Header, 0,sizeof(Header));
	Header.ImageType=2;
	Header.ImageDescriptor=8;
	Header.Width=SirkaBitmapy;
	Header.Height=VyskaBitmapy;
	Header.PixelDepth=32;
	fwrite(&Header,1,sizeof(Header),f);
	fwrite(Bitmap,1,sizeof(DWORD)*SirkaBitmapy*VyskaBitmapy,f);
	delete[] Bitmap;
	fclose(f);


}

int main(int argc, char* argv[])
{
	if (argc<6)
	{
		printf("dataacquire outputtype scale_d scale_s segmentfile.seg outfile.txt [optional params]\n");
		printf("outputtype:\n");
		printf("t..text \n");
		printf("i..image .. optional params: [color] [lefttop coords DxS rightbottom coords DxS]  \n");
		return -1;
	}


	string OutputType=argv[1];
	string InFileName=argv[4];
	string OutFileName=argv[5];

	double ScaleS=1.0f;
	double ScaleD=1.0f;

	ScaleD=atof(argv[2]);
	ScaleS=atof(argv[3]);

	
	//nacteme vstupni data
	LoadGPSSegmentFilter LFilter(InFileName);	
	GPSPointBuffer::Holder NullHolder;
	GPSPointBuffer::Holder LoadedData;
	LFilter.ProcessBuffer(NullHolder,LoadedData);

	if (LoadedData->Data.size()==0)
	{
		printf("no data in segment\n");
		return -2;
	}


	//podle pozadovaneho vystupu zavolame prislusnou funkci pro jeho vygenerovani
	if (OutputType=="t") GenerateTextOutput(LoadedData,OutFileName,ScaleS,ScaleD);
	if (OutputType=="i") 
	{
		DWORD Color=0xffffff;
		if (argc>=7) Color=atoi(argv[6]);

		if (argc>=11) 
		{
			double LeftTopD=atof(argv[7]);
			double LeftTopS=atof(argv[8]);
			double RightBottomD=atof(argv[9]);
			double RightBottomS=atof(argv[10]);

			GenerateImageOutput(LoadedData,OutFileName,ScaleS,ScaleD,Color,true,LeftTopD,LeftTopS,RightBottomD,RightBottomS);
		}
		else
			GenerateImageOutput(LoadedData,OutFileName,ScaleS,ScaleD,Color,false);
	}
	return 0;
}
