#include "dataparser.h"
#include <stdlib.h>

#define PNEW

#define CMD_CHAR 0x100
#define IS_CMD_CHAR(x) ((x)&CMD_CHAR)
#define GET_CMD(x) ((x)&0xff)


#define CMD_END				0
#define CMD_LEFTBRACKET_H	1
#define CMD_RIGHTBRACKET_H	2
#define CMD_LEFTBRACKET_W	3
#define CMD_RIGHTBRACKET_W	4
#define CMD_LEFTBRACKET		5
#define CMD_RIGHTBRACKET	6
#define CMD_QUESTION		7
#define CMD_STAR			8
#define CMD_PLUS			9
#define CMD_COMPLEMENT		10
#define CMD_ERROR			20


#define CMD(x) (CMD_CHAR|(x))


static int StringReadChar(ConstCStr &Cislo)
{
unsigned char Character=*((unsigned char *)Cislo);
if (Character) Cislo++; //Abychom nepresli za konec retezce
switch (Character)
	{
	case 0:return CMD(CMD_END);
	case '[':return CMD(CMD_LEFTBRACKET_H);
	case ']':return CMD(CMD_RIGHTBRACKET_H);
	case '{':return CMD(CMD_LEFTBRACKET_W);
	case '}':return CMD(CMD_RIGHTBRACKET_W);
	case '(':return CMD(CMD_LEFTBRACKET);
	case ')':return CMD(CMD_RIGHTBRACKET);
	case '?':return CMD(CMD_QUESTION);
	case '*':return CMD(CMD_STAR);
	case '+':return CMD(CMD_PLUS);
	case '^':return CMD(CMD_COMPLEMENT);
	case '\\':break; //osetrime venku
	default:return Character; //Normalni znak->rovnou ho vratime
	}
//zde budeme obsluhovat escapovane znaky
Character=*Cislo;
if (!Character) return CMD(CMD_ERROR); //za escapem byl konec retezce->chyba
Cislo++; 
return Character; //znak byl uvozen escape sekvenci->je to normalni znak
}

static int StringLookAheadChar(ConstCStr Cislo)
{
unsigned char Character=*((unsigned char *)Cislo);
if (Character) Cislo++; //Abychom nepresli za konec retezce
switch (Character)
	{
	case 0:return CMD(CMD_END);
	case '[':return CMD(CMD_LEFTBRACKET_H);
	case ']':return CMD(CMD_RIGHTBRACKET_H);
	case '{':return CMD(CMD_LEFTBRACKET_W);
	case '}':return CMD(CMD_RIGHTBRACKET_W);
	case '(':return CMD(CMD_LEFTBRACKET);
	case ')':return CMD(CMD_RIGHTBRACKET);
	case '?':return CMD(CMD_QUESTION);
	case '*':return CMD(CMD_STAR);
	case '+':return CMD(CMD_PLUS);
	case '^':return CMD(CMD_COMPLEMENT);
	case '\\':break; //osetrime venku
	default:return Character; //Normalni znak->rovnou ho vratime
	}
//zde budeme obsluhovat escapovane znaky
Character=*Cislo;
if (!Character) return CMD(CMD_ERROR); //za escapem byl konec retezce->chyba
Cislo++; 
return Character; //znak byl uvozen escape sekvenci->je to normalni znak
}


//syntaxe prikazoveho stringu, matchovani je hladove->"zere dokud muze"
//[abc] - na danem miste se muze vyskytovat libovolny z techto znaku
//[^abc] - na danem miste se muze vyskytovat libovolny znak krome abc
//+ predchazejici znak muze byt nekolikrat opakovan, nejmene vsak jednou
//* predchazejici znak muze byt nekolikrat opakovan nebo nemusi byt uveden
//? libovolny znak
//(vyraz) - lze kombinovat s +* pro opakovani celeho vnitrniho vyrazu->nelze vnorovat
//		  - retezec a pokracovani NESMI zacinat stejnym znakem
//{vyraz} - string, ktery matchuje s vyrazem se ulozi do vystupniho stringu 
//	        s indexem odpovidajicim poctu {}, ktere predchazeji
//			tj. pri maskce {[abc]}{[de]} a retezci aaaabbcbeeed
//			bude na vystupu "aaaabbcb","eeed"
//\\ - escape, prevede libovolny z predchazejicich znaku na normalni
//vnorovani zavorek stejneho typu neni povoleno a neni povoleno vnorit () nebo {} do []

//abcd  - matchuje s retezcem "abcd"
//a[bc]d - matchuje s retezcem "abd" nebo "acd"
//a[bc]*d - matchuje s retezcem "abbcd", "acd", "accccbcd", ....
//ab*d - matchuje s retezcem "abbd", "ad", "abbbbbbbd", .... !!!Pozor na b*b ->bude matchovat b ale ne s bb
//ab+d - matchuje s retezcem "abbd",  "abbbbbbbd", ale ne "ad"
//a?d     - matchuje s retezcem "a'cokoliv'd"

#include <windows.h>

static bool LogError(const char *Cause)
{
OutputDebugString(Cause);
OutputDebugString("\n");
return false;
}

bool DataParser::BuidGraph(ConstCStr CmdString)
{
Generated=0;
Koren.Clean();
if (!CmdString) CmdString=CmdStr.c_str();
if (!CmdString) return LogError("Prazdny retezec");
int x,y;
ParserGraphItem *Current=&Koren;
ParserGraphItem *VyrazRepeatStart=NULL; //Kde zacina vyraz ohraniceny (
ParserGraphItem *VyrazRepeatStartNext=NULL; //prvek za prvnim symbolem vyrazu
ParserGraphItem *LastCurrent=NULL; //prvek current pred prepnutim pokracovani, ktere bylo zpusobeno ()*
								   
bool Inside_VyrazRepeat=false;
bool Inside_Vyraz=false;
bool Inside_Multiselect=false;
int NextOutputIndex=1;  //0 znamena->zadny vyraz

char EnableMask[256]; //zde budeme uchovavat pracovni info o aktivovanych prvcich
bool InvertEnableMask=false; //jestli berem doplnek masky

memset(EnableMask,0,sizeof(EnableMask));

enum GenMode {GM_NORMAL,GM_STAR,GM_PLUS};
GenMode NextGenMode=GM_NORMAL;

while (1)
	{
	//projdeme cely datovy retezec a vytvorime strom
	int LaC;
	int c=StringReadChar(CmdString);
	if (c==CMD(CMD_END)) break;
	if (IS_CMD_CHAR(c))
		switch (GET_CMD(c))
			{
			case CMD_ERROR:
				return LogError("chyba syntaxe ridiciho stringu, ukoncen za '\'");
			case CMD_LEFTBRACKET_H:
				if (Inside_Multiselect)
					return LogError("chyba syntaxe ridiciho stringu, vnorene []");
				Inside_Multiselect=true;
				memset(EnableMask,0,sizeof(EnableMask));
				InvertEnableMask=false;
				continue;
			case CMD_RIGHTBRACKET_H:
				if (!Inside_Multiselect)
					return LogError("chyba syntaxe ridiciho stringu, ] bez [");
				Inside_Multiselect=false;
				break;
			case CMD_LEFTBRACKET_W:
				if (Inside_Vyraz)
					return LogError("chyba syntaxe ridiciho stringu, vnorene {}");
				if (Inside_Multiselect)
					return LogError("chyba syntaxe ridiciho stringu, uvnitr [{]");
				Inside_Vyraz=true;
				continue;
			case CMD_RIGHTBRACKET_W:
				if (Inside_Multiselect)
					return LogError("chyba syntaxe ridiciho stringu,  uvnitr [}]");
				if (!Inside_Vyraz)
					return LogError("chyba syntaxe ridiciho stringu, } bez {");
				NextOutputIndex++;
				Inside_Vyraz=false;
				continue;				


			case CMD_LEFTBRACKET:
				if (Inside_VyrazRepeat)
					return LogError("chyba syntaxe ridiciho stringu, vnorene ()");
				if (Inside_Multiselect)
					return LogError("chyba syntaxe ridiciho stringu, uvnitr [(]");
				VyrazRepeatStart=Current;
				VyrazRepeatStartNext=NULL;
				Inside_VyrazRepeat=true;
				continue;
			case CMD_RIGHTBRACKET:
				if (Inside_Multiselect)
					return LogError("chyba syntaxe ridiciho stringu,  uvnitr [)]");
				if (!Inside_VyrazRepeat)
					return LogError("chyba syntaxe ridiciho stringu, ) bez (");								
				Inside_VyrazRepeat=false;
				LaC=StringLookAheadChar(CmdString); //mrkneme, co bude nasledovat
				if (LaC==CMD(CMD_PLUS) || LaC==CMD(CMD_STAR)) 
					{
					//pro "0< staci privest specialni linky z konce na zacatek
					//vedeme linky do startnext pro vsechny znaky pro ktere vedou linky z start do startnext
					for (int x=0;x<256;x++)
						if (VyrazRepeatStart->Childs[x]==VyrazRepeatStartNext)
							{
							if (Current->ReleaseChild[x]) SAFE_REL(Current->Childs[x]);
							Current->Childs[x]=VyrazRepeatStartNext;
							Current->ReleaseChild[x]=0;
							Current->StoreCodeIndex[x]=VyrazRepeatStart->StoreCodeIndex[x];
							}					
					}				
				if (LaC==CMD(CMD_STAR)) 
					{					
					//pro 0<= budeme v dalsim navazovani pokracovat z bodu start					
					LastCurrent=Current;					
					Current=VyrazRepeatStart;
					}				
				continue;				
			case CMD_COMPLEMENT:
				if (!Inside_Multiselect)
					return LogError("chyba syntaxe ridiciho stringu, ^ mimo []");
				for (y=0;y<256;y++)
					if (EnableMask[y])
						return LogError("chyba syntaxe ridiciho stringu, ^ neni prvnim znakem uvnitr []");
				InvertEnableMask=true;
				continue;		
			case CMD_QUESTION:
				for (y=0;y<256;y++) EnableMask[y]=1;
				break;
			case CMD_STAR:
				if (Inside_Multiselect)
					return LogError("chyba syntaxe ridiciho stringu, * uvnitr []");
				continue; //prvek uz byl zpracovan v predchozim kroku
			case CMD_PLUS:
				if (Inside_Multiselect)
					return LogError("chyba syntaxe ridiciho stringu, + uvnitr []");
				continue; //prvek uz byl zpracovan v predchozim kroku
			default:				
				return LogError("chyba syntaxe ridiciho stringu");
			}	
	if (!IS_CMD_CHAR(c)) EnableMask[c]=1;
	LaC=StringLookAheadChar(CmdString); //mrkneme, co bude nasledovat

	if (Inside_Multiselect && (LaC==CMD(CMD_STAR) || LaC==CMD(CMD_PLUS)))
			return LogError("chyba syntaxe ridiciho stringu * nebo + uvnitr []");
	if (LaC==CMD(CMD_STAR)) NextGenMode=GM_STAR;
	if (LaC==CMD(CMD_PLUS)) NextGenMode=GM_PLUS;

	if (!Inside_Multiselect)
		{
		//nejsme uvnitr [] takze rovnou generujeme prechod
		if (InvertEnableMask) //chceme doplnek
			for (x=0;x<256;x++) EnableMask[x]^=1; //prohodime priznaky 

		int EnableCount=0;
		for (x=0;x<256;x++) EnableCount+=EnableMask[x]; //spocitame pocet "zapnutych" znaku
		if (!EnableCount)
			return LogError("chyba masky, zadne prechody, retezec obsahoval neco jako [] nebo [^vsechno] ci [^?]");

		PNEW;ParserGraphItem *Next=new ParserGraphItem;
		Next->AddRef();
		if (!Next) return LogError("malo pameti");

		int StoreIndex=0;
		if (Inside_Vyraz) StoreIndex=NextOutputIndex;
		
		switch (NextGenMode)
			{
			case GM_NORMAL:
				//normalni prechod
				for (x=0;x<256;x++)
					if (EnableMask[x])						
						{
						if (Current->ReleaseChild[x]) SAFE_REL(Current->Childs[x]);
						Next->AddRef();
						Current->Childs[x]=Next;
						Current->ReleaseChild[x]=1;
						Current->StoreCodeIndex[x]=StoreIndex;
						if (LastCurrent) //existuje bocni vetev, musime se napojit i na ni
							{
							Next->AddRef();
							LastCurrent->Childs[x]=Next;
							LastCurrent->ReleaseChild[x]=1;
							LastCurrent->StoreCodeIndex[x]=StoreIndex;
							}
						}
				Current=Next;				
				LastCurrent=NULL;
				break;
			case GM_STAR:
				//hvezdickovy prechod->vytvorime cyklus
				for (x=0;x<256;x++)
					if (EnableMask[x])						
						{
						if (Current->ReleaseChild[x]) SAFE_REL(Current->Childs[x]);						
						Current->Childs[x]=Current;
						Current->ReleaseChild[x]=0;
						Current->StoreCodeIndex[x]=StoreIndex;
						if (LastCurrent) //existuje bocni vetev, musime se napojit i na ni
							{							
							LastCurrent->Childs[x]=LastCurrent;
							LastCurrent->ReleaseChild[x]=0; //napojujeme se na sebe
							LastCurrent->StoreCodeIndex[x]=StoreIndex;
							}						
						}				
				//zde last current zustava protoze jsme vrchol jeste neopusitli
				break;
			case GM_PLUS:
				//plusovy prechod->nejdrive normalni krok a pak cyklus				
				for (x=0;x<256;x++)
					if (EnableMask[x])						
						{
						//normalni krok
						if (Current->ReleaseChild[x]) SAFE_REL(Current->Childs[x]);
						Next->AddRef();
						Current->Childs[x]=Next;
						Current->ReleaseChild[x]=1;
						Current->StoreCodeIndex[x]=StoreIndex;

						//v novem bode vytvorime cyklus. Prvek pro novy bod je cerstve alokovany->neni treba nic uvolnovat
						Next->Childs[x]=Next;
						Next->ReleaseChild[x]=0;
						Next->StoreCodeIndex[x]=StoreIndex;

						if (LastCurrent) //existuje bocni vetev, musime se napojit i na ni
							{		
							Next->AddRef();
							LastCurrent->Childs[x]=Next;
							LastCurrent->ReleaseChild[x]=1;
							LastCurrent->StoreCodeIndex[x]=StoreIndex;
							}												
						}
				Current=Next;											
				LastCurrent=NULL;
				break;
					
			}

		if (Inside_VyrazRepeat && VyrazRepeatStartNext==NULL) VyrazRepeatStartNext=Next;
		NextGenMode=GM_NORMAL;
		SAFE_REL(Next);
		memset(EnableMask,0,sizeof(EnableMask));
		InvertEnableMask=false;		
		
		}
	}

if (Inside_Multiselect || Inside_Vyraz || Inside_VyrazRepeat) return LogError("syntax error -> neukoncena nejaka zavorka");
//strom je hotovy

if (LastCurrent) LastCurrent->ValidString=1; //Existuje jeste konecna vedlejsi vetev->ukoncime ji take

Current->ValidString=1;;
Generated=1;
return true;
}


bool DataParser::ParseString(ConstCStr String,ParseCallback Callback,void *CallbackData)
{
if (!Generated) 
	if (!BuidGraph()) return false;
string Output="";	
ParserGraphItem *Current=&Koren;
int LastOutputIndex=0;
while (*String) //Dokud nejsme na konci stringu
	{
	int C=*((unsigned char *)(String++));
	if (!Current->Childs[C]) return false; //ve strome neni link pro dany symbol->slovo neni z jazyka
	int Index=Current->StoreCodeIndex[C];
	if (Index!=LastOutputIndex && LastOutputIndex) //Index se od minula zmenil a minule byl nenulovy->je hotovy nejaky string
		{
		if (Callback) Callback(LastOutputIndex,Output,CallbackData); //posleme ho do calbacku, pokud je definovan
		Output="";	//pripravime prazdny string pro nova data		
		}		
	LastOutputIndex=Index; 
	if (Index) Output+=C; //znak pridame na konec stringu
	Current=Current->Childs[C];	
	}
if (LastOutputIndex && Callback) Callback(LastOutputIndex,Output,CallbackData); //posleme ho do calbacku, pokud je definovan
return Current->ValidString!=0;
}


static void ExParseCallback(int Index,string &s,void *UserData)
{
INT_to_STRING *Map=(INT_to_STRING *)UserData;
Map->insert(INT_to_STRING::value_type(Index-1,s));
}

bool DataParser::ParseStringEx(ConstCStr String,INT_to_STRING &Output)
{
Output.clear();
bool rv=ParseString(String,ExParseCallback,&Output);
if (!rv) Output.clear();
return rv;
}