#include "archive.h"
#include "archivestl.h"

void Archive::RegisterClass(Archivable *ClassFactory)
{
	assert(!"Archive::RegisterClass called");
}

void Archive::InitArchive(ArchiveDataSource *DS,bool ForRead)
{
	DataSource=DS;
	OperationMode=ForRead?READ:WRITE;
}
void Archive::DoneArchive(void)
{
}



template<class X>
static Archive &GenericArchiveOp(Archive &arch,X &Value)
{
	arch.Raw(&Value,sizeof(Value));
	return arch;
}

Archive &Archive::operator<<=(bool &Value)
{
	return GenericArchiveOp(*this,Value);
}

Archive &Archive::operator<<=(int &Value)
{
	return GenericArchiveOp(*this,Value);
}

Archive &Archive::operator<<=(unsigned long &Value)
{
	return GenericArchiveOp(*this,Value);
}

Archive &Archive::operator<<=(float &Value)
{
	return GenericArchiveOp(*this,Value);
}

Archive &Archive::operator<<=(long double &Value)
{
	return GenericArchiveOp(*this,Value);
}


Archive &Archive::operator<<=(char &Value)
{
	return GenericArchiveOp(*this,Value);
}



void Archive::Raw(void *Buff,int Size)
{
	if (OperationMode==WRITE) DataSource->Write(Buff,Size);
	if (OperationMode==READ) DataSource->Read(Buff,Size);
}

void Archive::Raw(const void *Buff,int Size)
{
	assert(OperationMode!=READ);
	if (OperationMode==WRITE)
		DataSource->Write(Buff,Size);
}

int Archive::GetOffset(void)
{
	return DataSource->GetPos();
}


//ExtendedArchive je mimo hlavni header aby byl chranen pred stl
struct ArchiveRegisteredClass
{
	string Name;
	Archivable *ClassFactory;
	int IdInFile;	
	ArchiveRegisteredClass(string N,Archivable *CF,int Id) {Name=N;ClassFactory=CF;IdInFile=Id;}
};

struct ArchiveObjectInstance
{
	Archivable *AdresaObjektu;
	bool StaticSource;
	int FileOffset;
};

class ExtendedArchive:public Archive
{
	
	 map<string,ArchiveRegisteredClass> MapOfRegisteredClassesByName; //pouzivano pri save
	 map<int,ArchiveRegisteredClass> MapOfRegisteredClassesById;  //pouzivano pri load pro nalezeni informace o class factory

	 map<Archivable *,ArchiveObjectInstance> MapAddressToObjectInstance; //prevod Addr->offset v souboru
	 map<int,ArchiveObjectInstance> MapOffsetToObjectInstance; //prevod offset->addr objektu
	 public:
	 virtual void RegisterClass(Archivable *ClassFactory);
	 virtual void InitArchive(ArchiveDataSource *DS,bool ForRead); //volat po vytvoreni archivu
	 //zakladni operace
	 virtual Archive &operator<<=(Archivable &Value);
	 virtual Archive &operator<<=(Archivable *&Value);
};

Archive *Archive::CreateArchive(void)
{
	return new ExtendedArchive;
}


Archive &ExtendedArchive::operator<<=(Archivable &Value)
{
	if (DoLoad())
	{
		int ClassId;
		(*((Archive *)this))<<=ClassId;		
		map<int,ArchiveRegisteredClass>::iterator it=MapOfRegisteredClassesById.find(ClassId);		
		assert(it!=MapOfRegisteredClassesById.end()); //musi byt zaregistrovana classa
		assert(it->second.Name==Value.GetClassName()); //musi to byt stejna classa jako ma nas objekt

		int OffsetValue;
		(*((Archive *)this))<<=OffsetValue;
		if (OffsetValue)
		{
			//tato classa uz byla jednou nacitana
			map<int,ArchiveObjectInstance>::iterator it=MapOffsetToObjectInstance.find(OffsetValue);
			assert(it!=MapOffsetToObjectInstance.end()); //pokud uz byla nacitana tak musi byt v tabulce
			assert(it->second.AdresaObjektu==&Value); //protoze jsme staticky objekt tak jsme museli nacitat sebe (jinak chyba)
			assert(it->second.StaticSource); //prvni nacteni nebylo dynamickym objektem			
			return *this; //nema cenu znova nacitat protoze uz jsme jednou nacteni byli
		}
		else
		{
			//clasa jeste nebyla nactena->zaregistrujeme se
			ArchiveObjectInstance Tmp;
			Tmp.AdresaObjektu=&Value;
			Tmp.StaticSource=true;
			Tmp.FileOffset=GetOffset();
			MapOffsetToObjectInstance.insert(map<int,ArchiveObjectInstance>::value_type(Tmp.FileOffset,Tmp));
			try
			{
				int Version;
				(*((Archive *)this))<<=Version;
				bool Supported=Value.IsSupportedForLoad(Version);
				assert(Supported);
				Value.DoOperation(*this,Version); //provedeme vlastni nacteni
			}
			catch(...)
			{
				Value.Invalidate(); //odstraneni chybne nactenych dat
				throw;
			}
		}		
	}
	else
	{
		//delame save
		map<string,ArchiveRegisteredClass>::iterator fnd=MapOfRegisteredClassesByName.find(Value.GetClassName());
		assert(fnd!=MapOfRegisteredClassesByName.end());
		int ClassId=fnd->second.IdInFile; //zjistime id classy
		assert(ClassId);
		(*((Archive *)this))<<=ClassId;

		//zjistime jestli nahodou nedelame druhe save
		map<Archivable *,ArchiveObjectInstance>::iterator it=MapAddressToObjectInstance.find(&Value);
		if (it!=MapAddressToObjectInstance.end())
		{
			//uz jsme jednou ulozeni byli
			assert(it->second.StaticSource); //prvni ulozeni nesmel byt dynamicky objekt
			assert(it->second.AdresaObjektu=&Value); //prvni ulozeni musel byt tento objekt
			int Offset=it->second.FileOffset;
			(*((Archive *)this))<<=Offset; //zapiseme pouze offset
			return *this;
		}
		else
		{
			int Offset=0;
			(*((Archive *)this))<<=Offset; //zapiseme nulovy offset jako ze jsme novy objekt

			ArchiveObjectInstance Tmp;
			Tmp.AdresaObjektu=&Value;
			Tmp.StaticSource=true;
			Tmp.FileOffset=GetOffset();
			MapAddressToObjectInstance.insert(map<Archivable *,ArchiveObjectInstance>::value_type(&Value,Tmp));
			
			int Version=Value.GetSavedVersion();
			(*((Archive *)this))<<=Version;
			Value.DoOperation(*this,Version); //zapiseme objekt
		}
	}
	return *this;
}

Archive &ExtendedArchive::operator<<=(Archivable *&Value)
{
	if (DoLoad())
	{
		int ClassId;
		(*((Archive *)this))<<=ClassId;		
		if (ClassId==0)
		{
			//null pointer
			Value=NULL;
			return *this;
		}

		map<int,ArchiveRegisteredClass>::iterator it=MapOfRegisteredClassesById.find(ClassId);		
		assert(it!=MapOfRegisteredClassesById.end()); //musi byt zaregistrovana classa
		Archivable *Cf=it->second.ClassFactory;

		int OffsetValue;
		(*((Archive *)this))<<=OffsetValue;
		if (OffsetValue)
		{
			//tato classa uz byla jednou nacitana
			map<int,ArchiveObjectInstance>::iterator it=MapOffsetToObjectInstance.find(OffsetValue);
			assert(it!=MapOffsetToObjectInstance.end()); //pokud uz byla nacitana tak musi byt v tabulce
			Value=it->second.AdresaObjektu; //vratime pointer na puvodni nacteny objekt
			return *this; 
		}
		else
		{
			
			//vytvorime novou instnci			
			Value=Cf->ConstructClassInstance();
			assert(Value);

			//clasa jeste nebyla nactena
			ArchiveObjectInstance Tmp;			
			Tmp.StaticSource=false;
			Tmp.FileOffset=GetOffset();
			Tmp.AdresaObjektu=Value;
			try
			{
				int Version;
				(*((Archive *)this))<<=Version;
				bool Supported=Value->IsSupportedForLoad(Version);
				assert(Supported);

				Value->DoOperation(*this,Version); //provedeme vlastni nacteni									
				//zaregistrujeme classu
				MapOffsetToObjectInstance.insert(map<int,ArchiveObjectInstance>::value_type(Tmp.FileOffset,Tmp));
			}
			catch(...)
			{
				Value->Invalidate();
				Cf->DestructClassInstance(Value);
				Value=NULL;
				throw;
			}			
		}		
	}
	else
	{
		//delame save
		int ClassId=0;
		if (Value)
		{
			map<string,ArchiveRegisteredClass>::iterator fnd=MapOfRegisteredClassesByName.find(Value->GetClassName());
			assert(fnd!=MapOfRegisteredClassesByName.end());
			ClassId=fnd->second.IdInFile; //zjistime id classy
			assert(ClassId);
		}		
		(*((Archive *)this))<<=ClassId;

		if (!Value) return *this;

		//zjistime jestli nahodou nedelame druhe save
		map<Archivable *,ArchiveObjectInstance>::iterator it=MapAddressToObjectInstance.find(Value);
		if (it!=MapAddressToObjectInstance.end())
		{
			//uz jsme jednou ulozeni byli
			int Offset=it->second.FileOffset;
			(*((Archive *)this))<<=Offset; //zapiseme pouze offset
			return *this;
		}
		else
		{
			int Offset=0;
			(*((Archive *)this))<<=Offset; //zapiseme nulovy offset jako ze jsme novy objekt
			ArchiveObjectInstance Tmp;
			Tmp.AdresaObjektu=Value;
			Tmp.StaticSource=false;
			Tmp.FileOffset=GetOffset();
			MapAddressToObjectInstance.insert(map<Archivable *,ArchiveObjectInstance>::value_type(Value,Tmp));

			int Version=Value->GetSavedVersion();
			(*((Archive *)this))<<=Version;
			Value->DoOperation(*this,Version); //zapiseme objekt
		}
	}
	return *this;
}

void ExtendedArchive::RegisterClass(Archivable *ClassFactory)
{
	ArchiveRegisteredClass Tmp(ClassFactory->GetClassName(),ClassFactory,MapOfRegisteredClassesByName.size()+1);
	MapOfRegisteredClassesByName.insert(map<string,ArchiveRegisteredClass>::value_type(Tmp.Name,Tmp));
};


void ExtendedArchive::InitArchive(ArchiveDataSource *DS,bool ForRead)
{
	Archive::InitArchive(DS,ForRead);
	if (OperationMode==WRITE)
	{
		//zapiseme seznam registrovanych class
		int Count=MapOfRegisteredClassesByName.size();
		(*((Archive *)this))<<=Count; //zapiseme pocet
		map<string,ArchiveRegisteredClass>::iterator it;
		for (it=MapOfRegisteredClassesByName.begin();it!=MapOfRegisteredClassesByName.end();it++)
		{
			(*((Archive *)this))<<=it->second.IdInFile;
			(*((Archive *)this))<<=it->second.Name;
			int Ver=it->second.ClassFactory->GetSavedVersion();
			(*((Archive *)this))<<=Ver;
		}
	}
	else
	{
		//nacteme seznam registrovanych class
		int Count;
		(*((Archive *)this))<<=Count; //nacteme pocet
		MapOfRegisteredClassesById.clear();
		for (int x=0;x<Count;x++)
		{
			int Id;
			(*((Archive *)this))<<=Id;
			string Name;
			(*((Archive *)this))<<=Name;

			int Version;
			(*((Archive *)this))<<=Version;
			//najdeme podle jmena v seznamu registrovanyc
			map<string,ArchiveRegisteredClass>::iterator it=MapOfRegisteredClassesByName.find(Name);
			assert(it!=MapOfRegisteredClassesByName.end()); //musi to byt registrovana classa
		
			if (it!=MapOfRegisteredClassesByName.end())
			{
				//zjistime jestli je to podporovana verse
				bool IsSupportedVer=it->second.ClassFactory->IsSupportedForLoad(Version);
				assert(IsSupportedVer);
				ArchiveRegisteredClass Tmp=it->second;
			
				Tmp.IdInFile=Id; //nastavime ID nactene ze souboru
				MapOfRegisteredClassesById.insert(map<int,ArchiveRegisteredClass>::value_type(Id,Tmp)); //naregistrujeme se do mapy podle Id
			}
		}
	}

}
