#ifndef FILTERLIST_H
#define FILTERLIST_H


#include "filter.h"

template<class X>
class ProcessingFiltersList:public RefCounter,public list<RefCounterHolder<ProcessingFilter<X> > > 
{
	public:
	void ExecuteList(Buffer<X>::Holder InBuffer,Buffer<X>::Holder &OutBuffer)
	{
		iterator it;		
		Buffer<X>::Holder PreviousBuffer=InBuffer;
		for (it=begin();it!=end();it++)
		{
			Buffer<X>::Holder OBuffer;
			(*it)->ProcessBuffer(PreviousBuffer,OBuffer);
			PreviousBuffer=OBuffer;
		}
		OutBuffer=PreviousBuffer;
	}
};

template<class X>
class DisplayingOfProcessingFiltersList:public RefCounter
{
	Buffer<X>::Holder InputBuffer;
	RefCounterHolder<ProcessingFiltersList<X> > PreprocessingFilters;
	RefCounterHolder<DisplayingFilter<X> > FilterUsedForDisplay;
	
	Buffer<X>::Holder  CachedGraphResult;
	public:

	DisplayingOfProcessingFiltersList(void) {}
	DisplayingOfProcessingFiltersList(RefCounterHolder<ProcessingFiltersList<X> > FiltersList,RefCounterHolder<DisplayingFilter<X> > DisplayFlt)
	{
		SetList(FiltersList);
		SetDisplayFilter(DisplayFlt);
	}

	void SetList(RefCounterHolder<ProcessingFiltersList<X> > FiltersList)
	{
		PreprocessingFilters=FiltersList;
		CachedGraphResult=NULL;
	}

	RefCounterHolder<ProcessingFiltersList<X> > GetList(void)
	{
		return PreprocessingFilters;
	}

	void SetDisplayFilter(RefCounterHolder<DisplayingFilter<X> > DisplayFlt)
	{
		FilterUsedForDisplay=DisplayFlt;
	}

	RefCounterHolder<DisplayingFilter<X> > GetDisplayFilter(void)
	{
		return FilterUsedForDisplay;
	}

	void SetInputBuffer(Buffer<X>::Holder IBuffer)
	{
		InputBuffer=IBuffer;
		CachedGraphResult=NULL;
	}

	Buffer<X>::Holder GetInputBuffer(void)
	{
		return InputBuffer;
	}

	Buffer<X>::Holder GetCachedBuffer(void)
	{
		if (CachedGraphResult==NULL) Recalculate();
		return CachedGraphResult;
	}
	
	
	void Recalculate(void)
	{
		CachedGraphResult=NULL;
		if (PreprocessingFilters) 
		{
			PreprocessingFilters->ExecuteList(InputBuffer,CachedGraphResult);
			if (CachedGraphResult) CachedGraphResult->Recalculate(GPSPB_WHAT_DELKANAKRIVCE);
		}
	}

	void CalculateIfNotCached(void)
	{
		if (CachedGraphResult==NULL) Recalculate();
	}

	void ClearCache(void)
	{
		CachedGraphResult=NULL;
	}

	void Redraw(DisplayInterface<X>::Holder DisplayInterface)
	{
		if (CachedGraphResult==NULL) Recalculate();
		if (CachedGraphResult) FilterUsedForDisplay->DisplayBuffer(CachedGraphResult,DisplayInterface);
	}

};





#endif