#include "filter_deformmatch.h"

#define U180 3.14
#define U90 (U180/2)
#define U270 (U180*(3/2.0))
#define U360 (U180*2)
#define U0 0

FTYPE DeformationMatchingFilter::GetAngle(FTYPE x,FTYPE y)
{
	if (x==0 && y==0) return 0;
	return atan2(x,y)+3.14;
}

int DeformationMatchingFilter::GetQuadrant(FTYPE Uhel)
{
	while (Uhel>U360) Uhel-=360;
	while (Uhel<U0) Uhel+=360;
	if (U0<=Uhel && Uhel<U90) return 0;
	if (U90<=Uhel && Uhel<U180) return 1;
	if (U180<=Uhel && Uhel<U270) return 2;
	if (U270<=Uhel && Uhel<U360) return 3;
	
	return 0;
}

#define MAX(a,b) (((a)>(b))?(a):(b))
#define MIN(a,b) (((a)<(b))?(a):(b))

FTYPE DeformationMatchingFilter::SpocitatOhybaciPraci(GPSPoint &SP1,GPSPoint &SP2,GPSPoint &SP3,GPSPoint &DP1,GPSPoint &DP2,GPSPoint &DP3)
{
	GPSPoint F0=Normalize(SP3-SP2);
	GPSPoint F1=Normalize(DP3-DP2);

	GPSPoint B0=Normalize(SP1-SP2);
	GPSPoint B1=Normalize(DP1-DP2);

	FTYPE Q0x=PointDot(F0,B0);
	FTYPE Q0y=PointMul(F0,B0);

	FTYPE Q1x=0.5*(PointDot(F1,B0)+PointDot(F0,B1));
	FTYPE Q1y=0.5*(PointMul(F1,B0)+PointMul(F0,B1));

	FTYPE Q2x=PointDot(F1,B1);
	FTYPE Q2y=PointMul(F1,B1);

	FTYPE AQ0=GetAngle(Q0x,Q0y);
	FTYPE AQ1=GetAngle(Q1x,Q1y);
	FTYPE AQ2=GetAngle(Q2x,Q2y);

	//zjistime zda dochazi k prechodu pres nulu
	//v ramci zjednoduseni se predpoklada ze k nemu muze dojit pouze tehdy kdyz je jeden z uhlu v intervalu <270,360) a druhy z intervalu <0,90)
	bool PrechodPresNulu=false;

	int QuadrantyProBody[3];
	QuadrantyProBody[0]=GetQuadrant(AQ0);
	QuadrantyProBody[1]=GetQuadrant(AQ1);
	QuadrantyProBody[2]=GetQuadrant(AQ2);

	if ((QuadrantyProBody[1]==0 || QuadrantyProBody[1]==3) &&	//prostredni bod je v 1 nebo 4 qadrantu
		((QuadrantyProBody[0]==0 && QuadrantyProBody[2]==3)  || 
		 (QuadrantyProBody[0]==3 && QuadrantyProBody[2]==0))		
		) PrechodPresNulu=true;


	
	FTYPE Uhel;

	if (!PrechodPresNulu) 
		Uhel=MAX(AQ2,AQ0)-MIN(AQ2,AQ0); //neprechazime pres nulu, bereme rozdil uhlu
	else
		Uhel=MIN(AQ2,AQ0)+U360-MAX(AQ2,AQ0); //prechazime pres nulu, bereme "wrapovany" rozdil uhlu

	
	FTYPE Pootoceni=fabs(PointDot(F0,F1)-1.0);
	if (Pootoceni<MinimalniPenaltovanePootoceni) Pootoceni=0.0;
	return AngleChangePenalty*Uhel+ZeroCrossingPenalty*(PrechodPresNulu?1.0f:0.0f)+PenaltaZaPootoceni*Pootoceni;	
}

FTYPE DeformationMatchingFilter::SpocitatNatahovaciPraci(GPSPoint &SP1,GPSPoint &SP2,GPSPoint &DP1,GPSPoint &DP2)
{
	FTYPE Dist1=GPSPoint::Distance(SP1,SP2);
	FTYPE Dist2=GPSPoint::Distance(DP1,DP2);
	FTYPE Dst=((1-Cs)*MIN(Dist1,Dist2)+Cs*MAX(Dist1,Dist2));
	if (Dst==0.0) Dst=0.00001;
	return (Ks*fabs(Dist1-Dist2))/Dst;
}

struct TableItem
{
	FTYPE Hodnota;
	enum DirType {FROMLEFT,FROMTOP,DIAGONAL};
	DirType Direction;
};


void DeformationMatchingFilter::MatchCurves(GPSPointBuffer::Holder Curve1,GPSPointBuffer::Holder Curve2,Buffer<CrossInfo<GPSPoint> >::Holder &MatchResult)
{
	int x,y;
	if (Curve1->Data.size()<=1 || Curve2->Data.size()<=1) return;
	int MaxX=Curve1->Data.size();
	int MaxY=Curve2->Data.size();
	TableItem *Pole=new TableItem[MaxY*MaxX];
	for (y=0;y<MaxY;y++) //pro jistotu vynulujeme
		for (x=0;x<MaxX;x++)
			Pole[y*MaxX+x].Hodnota=-1;
	Pole[0*MaxX+0].Hodnota=0;

	GPSPoint *BodyCurve1=new GPSPoint[MaxX+1]; //1 kuli pomocnemu bodu
	GPSPoint *BodyCurve2=new GPSPoint[MaxY+1];


	//pracovni kopie bodu pro rychly pristup
	GPSPointBuffer::BufferIterator it;
	for (x=0,it=Curve1->Data.begin();it!=Curve1->Data.end();it++,x++)
		BodyCurve1[x]=*it;

	for (x=0,it=Curve2->Data.begin();it!=Curve2->Data.end();it++,x++)
		BodyCurve2[x]=*it;


	
	GPSPoint VirtualStart1;
	GPSPoint VirtualStart2;
	GPSPoint VirtualEnd1;
	GPSPoint VirtualEnd2;
	//tyhle body jsou tam proto abychom meli jak zacit
	//predpokladame ze prvnim bodem prochazi rovna primka a interpolujeme do "minulosti"
	VirtualStart1=BodyCurve1[0]-(BodyCurve1[1]-BodyCurve1[0]); 
	VirtualStart2=BodyCurve2[0]-(BodyCurve2[1]-BodyCurve2[0]);

	//podobne s koncem
	VirtualEnd1=BodyCurve1[MaxX-1]-(BodyCurve1[MaxX-2]-BodyCurve1[MaxX-1]);
	VirtualEnd2=BodyCurve2[MaxY-1]-(BodyCurve2[MaxY-2]-BodyCurve2[MaxY-1]);
	BodyCurve1[MaxX]=VirtualEnd1;
	BodyCurve2[MaxY]=VirtualEnd2;

	
	for (x=1;x<MaxX;x++) 
	{
		Pole[0*MaxX+x].Hodnota=Pole[0*MaxX+(x-1)].Hodnota+
			SpocitatNatahovaciPraci(BodyCurve1[x-1],BodyCurve1[x],BodyCurve2[0],BodyCurve2[0])+
			SpocitatOhybaciPraci(BodyCurve1[x-1],BodyCurve1[x],BodyCurve1[x+1],VirtualStart2,BodyCurve2[0],BodyCurve2[1]);
		Pole[0*MaxX+x].Direction=TableItem::FROMLEFT;
	}
	for (y=1;y<MaxY;y++)
	{
		Pole[y*MaxX+0].Hodnota=Pole[(y-1)*MaxX+0].Hodnota+
			SpocitatNatahovaciPraci(BodyCurve1[0],BodyCurve1[0],BodyCurve2[y-1],BodyCurve2[y])+
			SpocitatOhybaciPraci(VirtualStart1,BodyCurve1[0],BodyCurve1[1],BodyCurve2[y-1],BodyCurve2[y],BodyCurve2[y+1]);
		Pole[y*MaxX+0].Direction=TableItem::FROMTOP;
	}


#define SETTOMIN(x,y,o) {if ((x).Hodnota>(y)) {assert((y)>=0);(x).Hodnota=(y);(x).Direction=(o);};}
	//kod
	for (y=1;y<MaxY;y++)
		for (x=1;x<MaxX;x++)
		{
			FTYPE Prace;
			FTYPE OhybaciPrace=0.0f;
			
			if (x<MaxX-1 && y<MaxY-1) //uvnitr
				OhybaciPrace=SpocitatOhybaciPraci(BodyCurve1[x-1],BodyCurve1[x],BodyCurve1[x+1],BodyCurve2[y-1],BodyCurve2[y],BodyCurve2[y+1]);
			if (x==MaxX-1 && y<MaxY-1) //pravy sloupec
				OhybaciPrace=SpocitatOhybaciPraci(BodyCurve1[x-1],BodyCurve1[x],VirtualEnd1,BodyCurve2[y-1],BodyCurve2[y],BodyCurve2[y+1]);
			if (x<MaxX-1 && y==MaxY-1) //dolni radek
				OhybaciPrace=SpocitatOhybaciPraci(BodyCurve1[x-1],BodyCurve1[x],BodyCurve1[x+1],BodyCurve2[y-1],BodyCurve2[y],VirtualEnd2);
			if (x==MaxX-1 && y==MaxY-1) //pravy dolni roh
				OhybaciPrace=SpocitatOhybaciPraci(BodyCurve1[x-1],BodyCurve1[x],VirtualEnd1,BodyCurve2[y-1],BodyCurve2[y],VirtualEnd2);
							
			//posun pouze v Segment1
			Prace=Pole[y*MaxX+(x-1)].Hodnota+OhybaciPrace;
			Prace+=SpocitatNatahovaciPraci(BodyCurve1[x-1],BodyCurve1[x],BodyCurve2[y],BodyCurve2[y]);
			Pole[y*MaxX+x].Hodnota=Prace;
			Pole[y*MaxX+x].Direction=TableItem::FROMLEFT;

			//posun pouze v Segment2
			Prace=Pole[(y-1)*MaxX+x].Hodnota+OhybaciPrace;
			Prace+=SpocitatNatahovaciPraci(BodyCurve1[x],BodyCurve1[x],BodyCurve2[y-1],BodyCurve2[y]);
			SETTOMIN(Pole[y*MaxX+x],Prace,TableItem::FROMTOP);

			//posun v obou segmentech
			Prace=Pole[(y-1)*MaxX+(x-1)].Hodnota+OhybaciPrace;
			Prace+=SpocitatNatahovaciPraci(BodyCurve1[x-1],BodyCurve1[x],BodyCurve2[y-1],BodyCurve2[y]);
			SETTOMIN(Pole[y*MaxX+x],Prace,TableItem::DIAGONAL);
		}
	
		
	//zacneme v pravo dole a generujeme "tridy ekvivalence"
	int CurrX=MaxX-1;
	int CurrY=MaxY-1;

	MatchResult=new Buffer<CrossInfo<GPSPoint> >;
	bool XChanged=true;
	bool YChanged=true;
	while (1)
	{
		
		bool InLimit=true;
		if (CurrX!=0 || CurrY!=0)
		{
			FTYPE LastVal=0.0f;
			switch(Pole[CurrY*MaxX+CurrX].Direction)
			{
				case TableItem::FROMLEFT:LastVal=Pole[CurrY*MaxX+(CurrX-1)].Hodnota;break;
				case TableItem::FROMTOP:LastVal=Pole[(CurrY-1)*MaxX+CurrX].Hodnota;break;
				case TableItem::DIAGONAL:LastVal=Pole[(CurrY-1)*MaxX+(CurrX-1)].Hodnota;break;		
			}		
			if (fabs(LastVal-Pole[CurrY*MaxX+CurrX].Hodnota)>MaxChangeAllowed) 
				InLimit=false; //prirazeni je v limitu
		}		

		if (XChanged && YChanged)
		{
			//zmena v obou krocich->je treba zalozit novou "tridu ekvivalence"
			if (InLimit || MatchResult->Data.size()==0)
			{				
				CrossInfo<GPSPoint> CGPS;			
				MatchResult->Data.push_front(CGPS); //zalozime novy prvek
				MatchResult->Data.front().Points1.push_back(BodyCurve1[CurrX]);
				MatchResult->Data.front().Points2.push_back(BodyCurve2[CurrY]);
			}
			XChanged=false;
			YChanged=false;
		}
		else
		{
			if (XChanged) 
			{
				if (InLimit)
					MatchResult->Data.front().Points1.push_back(BodyCurve1[CurrX]);
			}
			if (YChanged) 
			{
				if (InLimit)
					MatchResult->Data.front().Points2.push_back(BodyCurve2[CurrY]);
			}
		}

		if (CurrX==0 && CurrY==0) break;

		switch(Pole[CurrY*MaxX+CurrX].Direction)
		{
			case TableItem::FROMLEFT:CurrX--;XChanged=true;break;
			case TableItem::FROMTOP:CurrY--;YChanged=true;break;
			case TableItem::DIAGONAL:CurrX--;CurrY--;XChanged=YChanged=true;break;		
		}		
	}

	delete[] BodyCurve1;
	delete[] BodyCurve2;
	delete[] Pole;

}
