#include "filter_defaultmerge.h"

#include <map>
#include <list>

typedef list<GPSPoint> CorrespondingPoints;
typedef map<int,CorrespondingPoints> MapOfCP;
typedef map<int,GPSPoint> ResultsMap;

bool UpdateOdchylka(FTYPE &Odchylka,GPSPoint &ResultPoint,GPSPoint &PointToAdd) //vraci true pokud je rozptyl vetsi nez stavajici maximum
{
	FTYPE R=GPSPoint::Distance(ResultPoint,PointToAdd);
	R*=R;
	if (Odchylka<R) 
	{
		Odchylka=R;
		return true;
	}
	return false;
}



struct VazenaSouradnice //jedna hodnota souradnice s prislusnou vahou
{
	FTYPE Hodnota; //vlastni hodnota souradnice
	int CurveIndex; //index krivky do ktere patrime
};


static int __cdecl QSortComparator(const void *elem1, const void *elem2 )  //tidime podle hodnot
{
	FTYPE f=(((VazenaSouradnice *)elem1)->Hodnota)-(((VazenaSouradnice *)elem2)->Hodnota);
	if (f<0) return -1;
	if (f>0) return 1;
	return 0;
}

static int __cdecl QSortComparatorFTYPE(const void *elem1, const void *elem2 )  //tidime podle hodnot
{
	FTYPE f=(*(FTYPE *)elem1)-(*(FTYPE *)elem2);
	if (f<0) return -1;
	if (f>0) return 1;
	return 0;
}


void DefaultMergeFilter::GenerateResult(GPSPointBuffer::Holder PrimaryCurve,Buffer<MatchPair<GPSPoint> >::Holder Matches,GPSPointBuffer::Holder &ResultCurve)
{
	ResultCurve=NULL;
	if (PrimaryCurve->Data.size()==0) return;

	MapOfCP CorPoints; 
	int CurveIndex=1;

	//vytvoreni mnozin ekvivalentnich bodu
	Buffer<MatchPair<GPSPoint> >::BufferIterator it;
	for (it=Matches->Data.begin();it!=Matches->Data.end();it++,CurveIndex++)
	{
		Buffer<GPSPoint>::Holder SecondaryCurve=it->Curve;
		Buffer<CrossInfo<GPSPoint> >::Holder MatchData=it->MatchData;
		Buffer<CrossInfo<GPSPoint> >::BufferIterator it2;
		if (MatchData.GetValue()==NULL) continue;
		for (it2=MatchData->Data.begin();it2!=MatchData->Data.end();it2++)
		{		
			list<GPSPoint>::iterator it3,it4;
			for (it3=it2->Points1.begin();it3!=it2->Points1.end();it3++)
			{
				CorrespondingPoints &R=CorPoints[it3->IndexNaKrivce];
				//pro kazdy prvek ze seznamu 1 pridame do mapy jeden vrchol obsahujici zprumerovanou hodnotu vsech vrcholu v seznamu 2 ktere jsou k nemu vazane
				//cilem je dosahnout toho aby jedna krivka nemela prilis velky vliv jenom proto, ze ma prilis mnoho bodu
				
				int PocetSectenychHodnot=0;
				GPSPoint PrumernaHodnota;
				for (it4=it2->Points2.begin();it4!=it2->Points2.end();it4++)
				{
					int I=it4->IndexNaKrivce;
					GPSPointBuffer::BufferIterator Bit;
					for (Bit=SecondaryCurve->Data.begin();Bit!=SecondaryCurve->Data.end();Bit++)
						if (Bit->IndexNaKrivce==it4->IndexNaKrivce) break;

					if (Bit!=SecondaryCurve->Data.end())
					{
						PrumernaHodnota+=*Bit;
						PocetSectenychHodnot++;
					}
				}
				if (PocetSectenychHodnot)
				{
					PrumernaHodnota*=1.0/PocetSectenychHodnot;
					R.push_back(PrumernaHodnota);
					R.back().IndexKrivky=CurveIndex;
				}
			}
		}
	}

	//vygenerujeme body vysledne krivky
	ResultsMap Results;
	int LastId=-1;
	MapOfCP::iterator mit;	
	for (mit=CorPoints.begin();mit!=CorPoints.end();mit++)
	{
		GPSPointBuffer::BufferIterator Bit;
		for (Bit=PrimaryCurve->Data.begin();Bit!=PrimaryCurve->Data.end();Bit++)
			if (Bit->IndexNaKrivce==mit->first) break;
		if (Bit!=PrimaryCurve->Data.end())
		{
			//najdeme v kazde ose median a prvky do vzdalenosti urcene odchylkou od nej
			CorrespondingPoints &Cp=mit->second;
			int Count=1+Cp.size();
			VazenaSouradnice  *PositionsS=new VazenaSouradnice[Count];
			VazenaSouradnice  *PositionsD=new VazenaSouradnice[Count];

			//ulozime do poli souradnice vsech prvku v shluku
			int C=0;			
			VECT3D &V=*Bit;
			PositionsS[C].Hodnota=V.s;
			PositionsD[C].Hodnota=V.d;
			PositionsS[C].CurveIndex=0;
			PositionsD[C].CurveIndex=0;
			C++;
			CorrespondingPoints::iterator cit;
			for (cit=Cp.begin();cit!=Cp.end();cit++)
			{
				VECT3D &V=*cit;
				PositionsS[C].Hodnota=V.s;
				PositionsD[C].Hodnota=V.d;
				PositionsS[C].CurveIndex=cit->IndexKrivky;
				PositionsD[C].CurveIndex=cit->IndexKrivky;
				C++;
			}
			//setridime seznamy
			qsort(PositionsS,Count,sizeof(VazenaSouradnice),QSortComparator);
			qsort(PositionsD,Count,sizeof(VazenaSouradnice),QSortComparator);

			int MedianIndex=Count/2; //index medianu, souradnice mohou pochazet z ruznych vrcholu


			
			FTYPE MaxOdchD=MaximalniOdchylkaD;
			FTYPE MaxOdchS=MaximalniOdchylkaS;
			
			//pokud je MinimalTrackCountsS!=0 tak pripadne upravime MaximalniOdchylkaS tak, aby do teto vzdalenosti od medianu padl alespon zadany pocet tras
			if (MinimalTrackCountsS>0)
			{
				//do pole dame vsechny vzdalenosti od medianu
				FTYPE *OrderedByDist=new FTYPE[Count];						
				int q;
				for (q=0;q<Count;q++)
					OrderedByDist[q]=fabs(PositionsS[MedianIndex].Hodnota-PositionsS[q].Hodnota);

				//setridime podle vzdalenosti
				qsort(OrderedByDist,Count,sizeof(FTYPE),QSortComparatorFTYPE);

				int M=MinimalTrackCountsS;
				if (M>Count) M=Count;

				//vybereme M ty prvek serazene posloupnosti
				if (MaxOdchS<OrderedByDist[M-1]) MaxOdchS=OrderedByDist[M-1];
				delete[] OrderedByDist;
			}


			//pokud je MinimalTrackCountsD!=0 tak pripadne upravime MaximalniOdchylkaD tak, aby do teto vzdalenosti od medianu padl alespon zadany pocet tras
			if (MinimalTrackCountsD>0)
			{
				//do pole dame vsechny vzdalenosti od medianu
				FTYPE *OrderedByDist=new FTYPE[Count];						
				
				int q;
				for (q=0;q<Count;q++)
					OrderedByDist[q]=fabs(PositionsD[MedianIndex].Hodnota-PositionsD[q].Hodnota);

				//setridime podle vzdalenosti
				qsort(OrderedByDist,Count,sizeof(FTYPE),QSortComparatorFTYPE);

				int M=MinimalTrackCountsD;
				if (M>Count) M=Count;

				//vybereme M ty prvek serazene posloupnosti
				if (MaxOdchD<OrderedByDist[M-1]) MaxOdchD=OrderedByDist[M-1];
				delete[] OrderedByDist;
			}
			

			//nejdrive zjistime celkovy soucet vah prvku abychom zjistili nasobic potrebny pro to, abychom dostali soucet==1.0
			FTYPE SoucetVahS=0;
			FTYPE SoucetVahD=0;
			
			int q;
			for (q=0;q<Count;q++)
			{
				FTYPE DistS=fabs(PositionsS[MedianIndex].Hodnota-PositionsS[q].Hodnota);
				FTYPE DistD=fabs(PositionsD[MedianIndex].Hodnota-PositionsD[q].Hodnota);

				FTYPE VahaS=0;
				if (DistS<=MaxOdchS)
					VahaS=1.0;
				else
					if (DistS<=MaxOdchS+FadeoutS)
						VahaS=1.0-(DistS-MaxOdchS)/FadeoutS;
					else
						VahaS=0.0;
				SoucetVahS+=VahaS;

				FTYPE VahaD=0;
				if (DistD<=MaxOdchD)
					VahaD=1.0;
				else
					if (DistD<=MaxOdchD+FadeoutD)
						VahaD=1.0-(DistD-MaxOdchD)/FadeoutD;
					else
						VahaD=0.0;
				SoucetVahD+=VahaD;
			}


			//spocitame vazeny prumer
			FTYPE MKoefS=1.0/SoucetVahS;
			FTYPE MKoefD=1.0/SoucetVahD;
			
			FTYPE PrumerS=0.0f;
			FTYPE PrumerD=0.0f;
			for (q=0;q<Count;q++)
			{
				FTYPE DistS=fabs(PositionsS[MedianIndex].Hodnota-PositionsS[q].Hodnota);
				FTYPE DistD=fabs(PositionsD[MedianIndex].Hodnota-PositionsD[q].Hodnota);

				FTYPE VahaS=0;
				if (DistS<=MaxOdchS)
					VahaS=1.0;
				else
					if (DistS<=MaxOdchS+FadeoutS)
						VahaS=1.0-(DistS-MaxOdchS)/FadeoutS;
					else
						VahaS=0.0;
				FTYPE VahaD=0;
				if (DistD<=MaxOdchD)
					VahaD=1.0;
				else
					if (DistD<=MaxOdchD+FadeoutD)
						VahaD=1.0-(DistD-MaxOdchD)/FadeoutD;
					else
						VahaD=0.0;
				PrumerS+=PositionsS[q].Hodnota*VahaS*MKoefS;
				PrumerD+=PositionsD[q].Hodnota*VahaD*MKoefD;
			}

			GPSPoint Tmp;
			Tmp.InitByMetry(VECT3D(PrumerS,PrumerD,0));

			Results[mit->first]=Tmp;


			delete[] PositionsS;
			delete[] PositionsD;
		}
	}


	//doplnime chybejici prvky
	ResultsMap::iterator rmit;
	GPSPointBuffer::BufferIterator gpit;
	for (gpit=PrimaryCurve->Data.begin();gpit!=PrimaryCurve->Data.end();gpit++)
		if (Results.find(gpit->IndexNaKrivce)==Results.end())
		{
			//prvek neni zastoupen v zmergovanem poli->zkopirujeme ho z primarni krivky
			GPSPoint Tmp=*gpit;
			Results[gpit->IndexNaKrivce]=Tmp;
		}

		
	//vygenerujeme vysledny buffer
	int q=0;
	ResultCurve=new GPSPointBuffer;
	for (rmit=Results.begin();rmit!=Results.end();rmit++,q++)
		ResultCurve->Data.push_back(rmit->second);

	ResultCurve->Recalculate(GPSPB_WHAT_INDEXNAKRIVCE);
	ResultCurve->Recalculate(GPSPB_WHAT_DELKANAKRIVCE);
}