#ifndef GPSPOINT_H
#define GPSPOINT_H

#include "archive.h"
#include <math.h>
#include "buffer.h"


typedef long double FTYPE;

struct VECT3D
{
    FTYPE s,d,v;    

    VECT3D(void) {s=d=v=0.0f;}
	VECT3D(float S,float D,float V) {s=S;d=D;v=V;0.0f;}
    void operator+=(const VECT3D &o) {s+=o.s;d+=o.d;v+=o.v;}
    void operator-=(const VECT3D &o) {s-=o.s;d-=o.d;v-=o.v;}
    void operator*=(float Val) {s*=Val;d*=Val;v*=Val;}
    bool operator==(const VECT3D &o) {return s==o.s && d==o.d && v==o.v;}
    bool operator!=(const VECT3D &o) {return s!=o.s || d!=o.d || v!=o.v;}
};


class GPSPoint
{
     public:
     enum PointMode {UNINIT,STUPNE,METRY};
     PointMode Mode; //udava format originalnich dat, v soucasnosti podporovan pouze METRY

     VECT3D PoziceVeStupnich; //platna pokud Mode==STUPNE, pouze jako puvodni hodnota, pri vypoctech se nepouziva a ve vygenerovanych bodech neni
     VECT3D PoziceVMetrech;   //platna pokud Mode!=UNINIT, Pokud Mode==STUPNE tak je spocitana ze stupnu pri inicializaci

     FTYPE DelkaNaKrivce; //udava vzdalenost bodu od zacatku krivky, pouziva se pro vykreslovani v nekterych rezimech
     FTYPE DelkaKrivky; //celkova delka krivky ve ktere se nachazi tento bod, pouziva se pro vykreslovani v nekterych rezimech
 
     int IndexNaKrivce; //index bodu na krivce, indexovano od 0


     int IndexKrivky; //cislo sekundarni krivky do ktere bod patri, je pouzivano pri mergovani krivek
					  

     GPSPoint(void) {Mode=UNINIT;DelkaNaKrivce=DelkaKrivky=0.0;IndexNaKrivce=-1;IndexKrivky=0;}
     void InitByMetry(VECT3D Value);
     void InitByStupne(VECT3D Value); //zatim nepodporovano

     operator VECT3D&(void) {return PoziceVMetrech;}
     operator VECT3D(void) const {return PoziceVMetrech;}

     void operator+=(const GPSPoint &o) {PoziceVMetrech+=o.PoziceVMetrech;}
     void operator-=(const GPSPoint &o) {PoziceVMetrech-=o.PoziceVMetrech;}
     void operator*=(FTYPE Val) {PoziceVMetrech*=Val;}
     bool operator==(const GPSPoint &o) {return PoziceVMetrech==o.PoziceVMetrech;}
     bool operator!=(const GPSPoint &o) {return PoziceVMetrech!=o.PoziceVMetrech;}

     static FTYPE Distance(const GPSPoint &first,const GPSPoint &second);
};

void operator<<=(Archive &arch,VECT3D &p);
void operator<<=(Archive &arch,GPSPoint &p);

inline GPSPoint operator-(const GPSPoint &f,const GPSPoint &s)
{
	GPSPoint Out=f;
	Out-=s;
	return Out;
}


inline double PointDot(const GPSPoint &Frst,const GPSPoint &Scnd)
{
	return Frst.PoziceVMetrech.d*Scnd.PoziceVMetrech.d+Frst.PoziceVMetrech.s*Scnd.PoziceVMetrech.s;
}

inline double PointMul(const GPSPoint &Frst,const GPSPoint &Scnd)
{
	return Frst.PoziceVMetrech.d*Scnd.PoziceVMetrech.s-Scnd.PoziceVMetrech.d*Frst.PoziceVMetrech.s;
}

inline GPSPoint Normalize(const GPSPoint &p)
{
	FTYPE delka=sqrt(p.PoziceVMetrech.d*p.PoziceVMetrech.d+p.PoziceVMetrech.s*p.PoziceVMetrech.s);
	if (delka==0.0f) 
	{
		GPSPoint Out;
		Out.Mode=GPSPoint::METRY;
		Out.PoziceVMetrech.v=0;
		Out.PoziceVMetrech.s=0;
		Out.PoziceVMetrech.d=0;
		Out.DelkaNaKrivce=p.DelkaNaKrivce;
		return Out;
	}
	GPSPoint Out;
	Out.Mode=GPSPoint::METRY;
	Out.PoziceVMetrech.v=0;
	Out.PoziceVMetrech.s=(FTYPE)(p.PoziceVMetrech.s/delka);
	Out.PoziceVMetrech.d=(FTYPE)(p.PoziceVMetrech.d/delka);
	Out.DelkaNaKrivce=p.DelkaNaKrivce;
	return Out;
}


#define GPSPB_WHAT_DELKANAKRIVCE 1
#define GPSPB_WHAT_INDEXNAKRIVCE 2

class GPSPointBuffer:public Buffer<GPSPoint>
{ 
    public:
	string FileName; //jmeno souboru (bez cesty) ze ktereho buffer vzniknul, je plnen pri loadu a muze byt pouzit nekterym filtrem
	virtual void Recalculate(int What);
};


#endif