#ifndef REFCOUNT_H
#define REFCOUNT_H

#include <stdlib.h>
#include <assert.h>
#pragma warning (disable:4786)

#define SAFE_REL(x) ((x)?((x)->Release(),(x)=0):0)
#define SAFE_MAL(x,y) ((x)?(x):((x)=(y)))
#define SAFE_METHOD(x,met) ((x)?((x)->met):0)

#define RCSPECIF /*__forceinline*/

class RefCounter;
class RefCounterHolderBase
{
   public: //nejaky problem s prekladem RefCounterHolder je potomkem teto clasy ale pri protected rve prekladac
   RefCounterHolderBase *NextInList; 
   RefCounterHolderBase *PrevInList;


   public:
   RefCounterHolderBase(void) {NextInList=PrevInList=NULL;}
   virtual ~RefCounterHolderBase(void)
   {
		if (NextInList) NextInList->PrevInList=PrevInList;
		if (PrevInList) PrevInList->NextInList=NextInList;
   }

   virtual void ReleaseLink(RefCounter *Val)
   {
		if (NextInList) NextInList->PrevInList=PrevInList;
		if (PrevInList) PrevInList->NextInList=NextInList;
   } 

   friend class RefCounter;
};



class RefCounter
{
	protected:
	bool Static; //true pokud je polozka staticka, release ktere by zpusobilo jeji zruseni vyvola chybu
	int Count; //citac poctu referenci
	RefCounterHolderBase ChainRoot; //pointer na dalsi ze seznamu weak pointeru
	
    public:
	RCSPECIF RefCounter(bool Static=false) 
	{	
		Count=Static?1:0;
		RefCounter::Static=Static;
		ChainRoot.NextInList=ChainRoot.PrevInList=&ChainRoot;
	}
	
	virtual ~RefCounter(void) 
	{		
		//informujeme vsechny napojene objekty ze koncime
		while (ChainRoot.NextInList!=&ChainRoot)
		{
			RefCounterHolderBase *B=ChainRoot.NextInList;
			B->ReleaseLink(this);
		}
	}

	RCSPECIF int GetCounter(void) {return Count;};
	RCSPECIF void SetAsStatic(void) {Static=true;};
	RCSPECIF void SetAsDynamic(void) {Static=false;};

	RCSPECIF void AddRef(void) 
	{
		Count++;
	}

	RCSPECIF void Release(void) 
	{
		if (!(--Count)) 
		{
			assert(!Static);		
			delete this;
		}
	}

	
	RCSPECIF void InsertIntoList(RefCounterHolderBase *b)
	{				
		b->NextInList=ChainRoot.NextInList;
		b->PrevInList=&ChainRoot;
		b->NextInList->PrevInList=b;
		b->PrevInList->NextInList=b;
	}
};


template <class X,bool Weak=false>
class RefCounterHolder:public RefCounterHolderBase
	{
	X* Variable;
	public:
	RCSPECIF RefCounterHolder(void) {Variable=NULL;};
	RCSPECIF RefCounterHolder(const RefCounterHolder<X,false> &Co) 
	{
		Variable=NULL;
		SetValue((X *)Co.GetValue());
	}

	RCSPECIF RefCounterHolder(const RefCounterHolder<X,true> &Co) 
	{
		Variable=NULL;
		SetValue((X *)Co.GetValue());
	}

	RCSPECIF RefCounterHolder(X *Co) 
	{
		Variable=NULL;
		SetValue(Co);
	}
	RCSPECIF ~RefCounterHolder(void) {SetValue(NULL);};

	void DecValue(void) 
	{	   
		if (Variable) 
		{
			if (Variable->GetCount()>1) 
				Variable->Release();
			else
				SetValue(NULL);
		}
	}	

	RCSPECIF void InitToNULL(void) {Variable=NULL;};
	void SetValue(X *Co) 
	{
			if (Variable==Co) return; //Nastavujeme stejnou hodnotu->nic nemenime

			if (NextInList) NextInList->PrevInList=PrevInList; //odpojime se ze stareho seznamu
 			if (PrevInList) PrevInList->NextInList=NextInList;

			NextInList=PrevInList=NULL;

			X *LastValue=Variable; //precteme starou hodnotu 
			Variable=Co; //nastavime novou hodnotu.. 
			 //DULEZITE V pripade ze Release zavola zruseni 
			 //objektu nam bude zavolan ReleaseLink a ten tam 
			 //uz musi najit novou hodnotu aby to mohl rozlisit od 
			 //pripadu kdy mu nekdo zrusi objekt pod rukama
			if (!Weak)
			{
				//nejsme weak a tak cachrujeme s citaceme
				if (Variable) Variable->AddRef(); //zvetsime citac u nove hodnoty
				if (LastValue) LastValue->Release(); //zmensime citac u stare hodnoty
			}	    

			if (Variable) Variable->InsertIntoList(this);//pripojime se do seznamu u noveho objektu
	}

   	virtual void ReleaseLink(RefCounter *Val)
   	{
	   assert(Weak || Variable!=Val); 
		//pokud nejsme weak a doslo ke zruseni promene na kterou	
		//ukazujeme tak je to chyba

	   //v pripade silneho pointeru je mozne ze je tento kod
	   //volan pri nastavovani nove hodnoty a pak je volan 
	   //kdyz uz je Variable rovna nove hodnote a tedy ji musime nechat
	   if (Variable==Val)
	   {	   
	   		if (NextInList) NextInList->PrevInList=PrevInList;
	   		if (PrevInList) PrevInList->NextInList=NextInList;	  
			NextInList=PrevInList=NULL;
	   		Variable=NULL;			
	   }
	} 


	//operatory pro pristup k promenne
	RCSPECIF X *GetValue(void) {return Variable;};
	RCSPECIF const X *GetValue(void) const  {return Variable;};
	RCSPECIF X &operator=(X *Co) {SetValue(Co);return *Variable;};
	RCSPECIF RefCounterHolder<X> &operator=(const RefCounterHolder<X,false> &Co) {SetValue((X *)Co.GetValue());return *this;};
	RCSPECIF RefCounterHolder<X> &operator=(const RefCounterHolder<X,true> &Co) {SetValue((X *)Co.GetValue());return *this;};
	RCSPECIF operator X*(void) {return Variable;};
	RCSPECIF operator X*(void) const {return Variable;};
	RCSPECIF X *operator->(void) {return Variable;};
	RCSPECIF const X *operator->(void) const {return Variable;};
	RCSPECIF bool operator==(X *K) {return Variable==K;};
};

#endif